# CardSetExportHTML.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2007 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Export a card set to HTML."""

import gtk
from sutekh.core.SutekhObjects import PhysicalCardSet
from sutekh.core.CardSetHolder import CardSetWrapper
from sutekh.io.WriteArdbHTML import WriteArdbHTML
from sutekh.gui.PluginManager import SutekhPlugin
from sutekh.gui.SutekhDialog import do_complaint_error, do_exception_complaint
from sutekh.gui.SutekhFileWidget import ExportDialog
from sutekh.SutekhUtility import safe_filename


class CardSetExportHTML(SutekhPlugin):
    """Export a Card set to a 'nice' HTML file.

       We create a ElementTree that represents the XHTML file,
       and then dump that to file.
       This tries to match the HTML file produced by ARDB.
       """
    dTableVersions = {PhysicalCardSet: (4, 5, 6)}
    aModelsSupported = (PhysicalCardSet, "MainWindow")

    dGlobalConfig = {
        'HTML export mode': 'string(default=None)',
    }

    def get_menu_item(self):
        """Register on the Plugins Menu"""
        if not self.check_versions() or not self.check_model_type():
            return None

        if self._cModelType == "MainWindow":
            oPrefs = gtk.MenuItem("Export to HTML preferences")
            oSubMenu = gtk.Menu()
            oPrefs.set_submenu(oSubMenu)
            oGroup = None
            sDefault = self.get_config_item('HTML export mode')
            if sDefault is None:
                sDefault = 'Secret Library'
                self.set_config_item('HTML export mode',
                        sDefault)
            for sString, sVal in (("Add links to The Secret Library",
                'Secret Library'),
                    ("Add links to VTES Monger", 'Monger'),
                    ("Don't add links in the HTML file", 'None')):
                oItem = gtk.RadioMenuItem(oGroup, sString)
                if not oGroup:
                    oGroup = oItem
                oSubMenu.add(oItem)
                oItem.set_active(False)
                if sVal == sDefault:
                    oItem.set_active(True)
                oItem.connect("toggled", self.change_prefs, sVal)
            return ('File Preferences', oPrefs)

        oExport = gtk.MenuItem("Export to HTML")
        oExport.connect("activate", self.activate)
        return ('Export Card Set', oExport)

    def activate(self, _oWidget):
        """In response to the menu, create the dialog and run it."""
        oDlg = self.make_dialog()
        oDlg.run()
        self.handle_response(oDlg.get_name())

    def change_prefs(self, _oWidget, sChoice):
        """Manage the preferences (library to link to, etc.)"""
        sCur = self.get_config_item('HTML export mode')
        if sChoice != sCur:
            self.set_config_item('HTML export mode',
                    sChoice)

    # pylint: disable-msg=W0201
    # we define attributes outside __init__, but it's OK because of plugin
    # structure
    def make_dialog(self):
        """Create the dialog prompted for the filename."""
        oDlg = ExportDialog("Filename to save as", self.parent,
                '%s.html' % safe_filename(self.view.sSetName))
        oDlg.add_filter_with_pattern('HTML Files', ['*.html'])
        # pylint: disable-msg=E1101
        # vbox confuses pylint
        self.oTextButton = gtk.CheckButton("Include Card _Texts?")
        self.oTextButton.set_active(False)
        oDlg.vbox.pack_start(self.oTextButton, False, False)
        oDlg.show_all()
        return oDlg

    # pylint: enable-msg=W0201

    def handle_response(self, sFileName):
        """Handle the response to the dialog"""
        # pylint: disable-msg=E1101
        # SQLObject methods confuse pylint
        if sFileName is not None:
            # pylint: disable-msg=W0703
            # we do want to catch all exceptions here
            oCardSet = self.get_card_set()
            if not oCardSet:
                do_complaint_error("Unsupported Card Set Type")
                return
            bDoText = False
            if self.oTextButton.get_active():
                bDoText = True
            sLinkMode = self.get_config_item('HTML export mode')
            try:
                oWriter = WriteArdbHTML(sLinkMode, bDoText)
                fOut = file(sFileName, 'w')
                oWriter.write(fOut, CardSetWrapper(oCardSet))
                fOut.close()
            except Exception, oExp:
                sMsg = "Failed to open output file.\n\n%s" % oExp
                do_exception_complaint(sMsg)


plugin = CardSetExportHTML



# WhiteWolfTextParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# WhiteWolf Parser
# Copyright 2005, 2006 Simon Cross <hodgestar@gmail.com>
# Copyright 2007, 2011 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Text Parser for extracting cards from the online cardlist.txt."""

import re
import codecs
from sutekh.io.SutekhBaseHTMLParser import LogStateWithInfo
from logging import Logger
from sutekh.core.SutekhObjects import SutekhObjectMaker, csv_to_canonical


def strip_braces(sText):
    """Helper function for searching for keywords. Strip all {} tags from the
       text"""
    return sText.replace('{', '').replace('}', '')


# Card Saver
def _find_sect_and_title(aLines):
    """Search the first 2 lines of the card text for sect & title
       information.

       This is potentially brittle, since it had to rely on WW's
       standard text layout.
       """
    # pylint: disable-msg=R0912, R0915
    # R0912: Need to consider all the cases here
    # R0915: This is thus a long function
    # Card text for vampires is either Sect attributes. or Sect.
    sSect = None
    sTitle = None
    if aLines[0].find('Camarilla') != -1:
        sSect = 'Camarilla'
        if aLines[0].find('Camarilla primogen') != -1:
            sTitle = 'Primogen'
        elif aLines[0].find('Camarilla Prince of') != -1:
            sTitle = 'Prince'
        elif aLines[0].find('Justicar') != -1:
            # Since Justicar titles are of the form
            # 'Camarilla <Clan> Justicar'
            oJusticar = re.compile('Camarilla [A-Z][a-z]* Justicar')
            if oJusticar.search(aLines[0]) is not None:
                sTitle = 'Justicar'
        # Inner circle my be either Camariila Inner Circle or
        # Camarilla Clan Inner Circle
        # Hopefully this will go away sometime
        elif aLines[0].find('Inner Circle') != -1:
            if aLines[0].find('Camarilla Inner Circle') != -1:
                sTitle = 'Inner Circle'
            else:
                oInnerCircle = re.compile('Camarilla [A-Z][a-z]* Inner Circle')
                if oInnerCircle.search(aLines[0]) is not None:
                    sTitle = 'Inner Circle'
    elif aLines[0].find('Sabbat') != -1:
        sSect = 'Sabbat'
        if aLines[0].find('Sabbat Archbishop of') != -1:
            sTitle = 'Archbishop'
        elif aLines[0].find('Sabbat bishop') != -1:
            sTitle = 'Bishop'
        elif aLines[0].find('Sabbat priscus') != -1:
            sTitle = 'Priscus'
        elif aLines[0].find('Sabbat cardinal') != -1:
            sTitle = 'Cardinal'
        elif aLines[0].find('Sabbat regent') != -1:
            sTitle = 'Regent'
    elif aLines[0].find('Independent') != -1:
        sSect = 'Independent'
        # Independent titles are on the next line. Of the form
        # 'Name has X vote(s)'
        # pylint: disable-msg=W0704
        # error isn't fatal, so ignoring it is fine
        try:
            # Special cases 'The Baron' and 'Ur-Shulgi' mean we don't
            # anchor the regexp
            oIndTitle = re.compile('[A-Z][a-z]* has ([0-9]) vote')
            oMatch = oIndTitle.search(aLines[1])
            if oMatch is not None and not aLines[1].startswith('[MERGED]'):
                oMergedTitle = re.compile('MERGED.*has ([0-9]) vote')
                oMergedMatch = oMergedTitle.search(aLines[1])
                if oMergedMatch is not None and oMergedMatch.groups()[0] == \
                        oMatch.groups()[0]:
                    pass
                elif oMatch.groups()[0] == '1':
                    sTitle = 'Independent with 1 vote'
                elif oMatch.groups()[0] == '2':
                    sTitle = 'Independent with 2 votes'
                elif oMatch.groups()[0] == '3':
                    sTitle = 'Independent with 3 votes'
        except IndexError:
            pass
    elif aLines[0].find('Laibon') != -1:
        sSect = 'Laibon'
        if aLines[0].find('Laibon magaji') != -1:
            sTitle = 'Magaji'
    return sSect, sTitle


class CardDict(dict):
    """Dictionary object which holds the extracted card info."""

    # Usefule regex's
    oDisGaps = re.compile(r'[\\\/{}\&\s]+')
    oWhiteSp = re.compile(r'[{}\s]+')
    oDispCard = re.compile(r'\[[^\]]+\]$')
    oArtistSp = re.compile(r'[&;]')
    # Use regexp lookahead for the last '.', so it can anchor the next match
    # Use non-grouping parentheses so we ca can catch either . or the end of
    # the text
    # Defaults
    oCryptInfoRgx = re.compile(
            '[:\.] ([+-]\d) (bleed|strength|stealth|intercept)(?:(?=\.)|$)')
    dCryptProperties = {
            'black hand': re.compile('Sabbat\. Black Hand'),
            # Seraph has a special case
            'seraph': re.compile('Sabbat\. Black Hand(\.)? Seraph'),
            'infernal': re.compile('[.:] Infernal\.'),
            'red list': re.compile('\. Red List:'),
            'anarch': re.compile('\. Anarch:'),
            'scarce': re.compile('[.:] Scarce.'),
            'sterile': re.compile('[.:] Sterile.'),
            # Need the } to handle some of the errata'd cards
            'blood cursed': re.compile('[.:\}] \(?Blood [Cc]ursed'),
            'not for legal play': re.compile(
                'NOT FOR LEGAL PLAY|Added to the V:EKN banned list'),
            }

    # Properites we check for all library cards
    dLibProperties = {
            'not for legal play': re.compile(
                'NOT FOR LEGAL PLAY|Added to the V:EKN banned list'),
            }

    # Ally properties
    dAllyProperties = {
            # Red list allies are templated differently
            'red list': re.compile('\. Red List\.'),
            }
    oLifeRgx = re.compile('(Unique )?\[?(Gargoyle creature|[A-Za-z]+)\]?'
            ' with (\d) life\.')

    # equipment properties
    dEquipmentProperties = {
            'unique': re.compile('Unique (melee )?weapon|Unique equipment|'
                'represents a unique location|'
                'this is a unique location|'
                'as equipment (while|when) in play. (Haven. )?Unique.'),
            'location': re.compile('represents a (unique )?location|'
                'this is a (unique )?location'),
            'melee weapon': re.compile('[mM]elee weapon\.'),
            'cold iron': re.compile('weapon\. Cold iron\.'),
            'gun': re.compile('[wW]eapon[:,.] [gG]un\.'),
            'weapon': re.compile('[wW]eapon[:,.] [gG]un\.|[mM]elee weapon\.|'
                'Weapon.|Unique weapon.'),
            'vehicle': re.compile('Vehicle\.'),
            'haven': re.compile('Haven\.'),
            'electronic equipment': re.compile('Electronic equipment.'),
            }

    # master properties
    dMasterProperties = {
            # unique isn't very consistent
            'unique': re.compile('[Uu]nique [mM]aster|Master[:.] unique|'
                'Unique\.'),
            'trifle': re.compile('[mM]aster[:.] .*[tT]rifle'),
            'discipline': re.compile('Master: Discipline\.'),
            'out-of-turn': re.compile('Master: out-of-turn'),
            'location': re.compile('Master[:.] (unique )?[Ll]ocation'),
            'boon': re.compile('Boon\.'),
            'frenzy': re.compile('Frenzy\.'),
            'hunting ground': re.compile('\. Hunting [Gg]round'),
            'haven': re.compile('Haven\.'),
            'trophy': re.compile('Master\. Trophy'),
            'investment': re.compile('Master[.:] (unique )?[Ii]nvestment'),
            'archetype': re.compile('Master: archetype'),
            'watchtower': re.compile('Master: watchtower'),
            'title': re.compile('Title\.'),
            }

    # event properties
    dEventProperties = {
            'gehenna': re.compile('Gehenna\.'),
            'transient': re.compile('Transient\.'),
            'inconnu': re.compile('Inconnu\.'),
            'government': re.compile('Government\.'),
            'inquisition': re.compile('Inquisition\.'),
            }

    # Catch for non-specified card types
    dOtherProperties = {
            'unique': re.compile('Unique\.'),
            'boon': re.compile('Boon\.'),
            'watchtower': re.compile('Watchtower\.'),
            'frenzy': re.compile('Frenzy\.'),
            'title': re.compile('Title\.'),
            }

    # Special cases that aren't handled by the general code
    dAllyKeywordSpecial = {
            'Gypsies': ['1 stealth'],
            'Veneficti (Mage)': ['1 stealth'],
            'High Top': ['1 intercept'],
            'Ghoul Retainer': ['1 strength'],
            }
    # These vampires aren't templated as normal
    dCryptKeywordSpecial = {
            'Spider-Killer': {'stealth': 1},
            'Muaziz, Archon of Ulugh Beg': {'stealth': 1},
            'Rebekka, Chantry Elder of Munich': {'stealth': 1},
            }

    def __init__(self, oLogger):
        super(CardDict, self).__init__()
        self.oLogger = oLogger
        self._oMaker = SutekhObjectMaker()

    def _find_crypt_keywords(self, oCard):
        """Extract the bleed, strength & stealth keywords from the card text"""
        dKeywords = {'bleed': 1, 'strength': 1, 'stealth': 0, 'intercept': 0}
        # Correct special cases
        if self['name'] in self.dCryptKeywordSpecial:
            for sKeyword, iVal in \
                    self.dCryptKeywordSpecial[self['name']].iteritems():
                dKeywords[sKeyword] = iVal
        # Make sure we don't detect merged properties
        sText = strip_braces(self['text'].split('[MERGED]')[0])
        for sNum, sType in self.oCryptInfoRgx.findall(sText):
            dKeywords[sType] += int(sNum)
        for sType, iNum in dKeywords.iteritems():
            self._add_keyword(oCard, '%d %s' % (iNum, sType))
        # Check for "Black Hand", "Infernal", "Red List", "Seraph",
        for sKeyword, oRegexp in self.dCryptProperties.iteritems():
            oMatch = oRegexp.search(sText)
            if oMatch:
                self._add_keyword(oCard, sKeyword)
        # Add Non-Unique keyword
        if 'are not unique' in sText or 'Non-unique' in sText:
            self._add_keyword(oCard, 'non-unique')
        # Imbued are also mortals, so add the keyword
        if self['cardtype'] == 'Imbued':
            self._add_keyword(oCard, 'mortal')

    def _find_lib_life_and_keywords(self, oCard):
        """Extract ally and retainer life and strength & bleed keywords from
           the card text"""
        # Restrict ourselves to text before Superior disciplines
        sText = strip_braces(re.split('\[[A-Z]{3}\]', self['text'])[0])
        # Annoyingly not standardised
        oDetail1Rgx = re.compile('\. (\d strength), (\d bleed)[\.,]')
        oDetail2Rgx = re.compile('\. (\d bleed), (\d strength)[\.,]')
        oMatch = self.oLifeRgx.search(sText)
        if oMatch:
            # Normalise type
            if oMatch.group(1):
                self._add_keyword(oCard, 'unique')
            sType = oMatch.group(2).lower().replace(']', '')
            self._add_keyword(oCard, sType)
            self['life'] = oMatch.group(3)
            oDetail = oDetail1Rgx.search(sText)
            if not oDetail:
                oDetail = oDetail2Rgx.search(sText)
            if oDetail:
                self._add_keyword(oCard, oDetail.group(1))
                self._add_keyword(oCard, oDetail.group(2))
        self._find_card_keywords(oCard, self.dAllyProperties)
        if self['name'] in self.dAllyKeywordSpecial:
            for sKeyword in self.dAllyKeywordSpecial[self['name']]:
                self._add_keyword(oCard, sKeyword)

    def _find_card_keywords(self, oCard, dProps):
        """Find keywords for library cards"""
        sText = strip_braces(self['text'])

        def _do_match(sKeyword, oRegexp):
            """Helper to do the match"""
            oMatch = oRegexp.search(sText)
            if oMatch:
                self._add_keyword(oCard, sKeyword)

        for sKeyword, oRegexp in dProps.iteritems():
            _do_match(sKeyword, oRegexp)
        for sKeyword, oRegexp in self.dLibProperties.iteritems():
            _do_match(sKeyword, oRegexp)

    def _parse_text(self, oCard):
        """Parse the CardText for Sect and Titles"""
        # pylint: disable-msg=R0912
        # Complex set of conditions, so many branches
        sType = None
        if self.has_key('cardtype'):
            sTypes = self['cardtype']
            # Determine if we need to examine the card further based on type
            for sVal in sTypes.split('/'):
                if sVal in ['Vampire', 'Imbued', 'Ally', 'Retainer',
                        'Equipment', 'Master', 'Event']:
                    sType = sVal
        # Check for REFLEX card type
        if self['text'].find(' [REFLEX] ') != -1:
            if self.has_key('cardtype'):
                # append to card types
                self['cardtype'] += '/Reflex'
            else:
                self['cardtype'] = 'Reflex'
        if sType == 'Imbued' or sType == 'Vampire':
            self._find_crypt_keywords(oCard)
        elif sType == 'Ally' or sType == 'Retainer':
            self._find_lib_life_and_keywords(oCard)
        elif sType == 'Equipment':
            self._find_card_keywords(oCard, self.dEquipmentProperties)
        elif sType == 'Master':
            self._find_card_keywords(oCard, self.dMasterProperties)
        elif sType == 'Event':
            self._find_card_keywords(oCard, self.dEventProperties)
        else:
            self._find_card_keywords(oCard, self.dOtherProperties)
        if sType == 'Vampire':
            # Sect attributes: more text. Title is in the attributes
            aLines = strip_braces(self['text']).split(':')
            sSect, sTitle = _find_sect_and_title(aLines)
            # check if the vampire has flight (text ends has Flight [FLIGHT].)
            oFlightRexegp = re.compile('Flight \[FLIGHT\]\.')
            oMatch = oFlightRexegp.search(aLines[-1])
            if oMatch:
                if self.has_key('discipline'):
                    self['discipline'] += ' FLI'
                else:
                    self['discipline'] = 'FLI'
            if sSect is not None:
                self['sect'] = sSect
            if sTitle is not None:
                self['title'] = sTitle

    def _add_blood_shadowed_court(self, oCard):
        """Add Blood Shadowed Court to the expansion list if appropriate."""
        oCamVampPair = self._oMaker.make_rarity_pair('CE', 'Vampire')
        if oCamVampPair in oCard.rarity:
            oPair = self._oMaker.make_rarity_pair('BSC', 'BSC')
            if oPair not in oCard.rarity:
                # Don't duplicate entries
                oCard.addRarityPair(oPair)

    def _make_card(self, sName):
        """Create the abstract card in the database."""
        sName = self.oDispCard.sub('', sName)
        return self._oMaker.make_abstract_card(sName)

    def _add_expansions(self, oCard, sExp):
        """Add expansion information to the card, creating expansion pairs
           as needed."""
        aPairs = [x.split(':') for x in sExp.strip('[]').split(',') if x]
        aExp = []
        for aPair in aPairs:
            if len(aPair) == 1:
                aExp.append((aPair[0].strip(), 'NA'))
            elif aPair[1].strip().startswith('Promo-'):
                # Handle the TR:Promo special case
                aExp.append((aPair[1].strip(), 'NA'))
            else:
                aExp.append((aPair[0].strip(), aPair[1].strip()))

        for sExp, sRarSet in aExp:
            for sRar in sRarSet.split('/'):
                oPair = self._oMaker.make_rarity_pair(sExp, sRar)
                if oPair not in oCard.rarity:
                    oCard.addRarityPair(oPair)

    def _add_disciplines(self, oCard, sDis):
        """Add the list of disciplines to the card, creating discipline
           pairs as needed."""
        sDis = self.oDisGaps.sub(' ', sDis).strip()

        if sDis == '-none-' or sDis == '':
            return

        for sVal in sDis.split():
            if  sVal == sVal.lower():
                oPair = self._oMaker.make_discipline_pair(sVal, 'inferior')
            else:
                oPair = self._oMaker.make_discipline_pair(sVal, 'superior')
            if oPair not in oCard.discipline:
                oCard.addDisciplinePair(oPair)

    def _add_virtues(self, oCard, sVir):
        """Add the list of virtues to the card."""
        sVir = self.oDisGaps.sub(' ', sVir).strip()

        if sVir == '-none-' or sVir == '':
            return

        for sVal in sVir.split():
            oVirt = self._oMaker.make_virtue(sVal)
            if oVirt not in oCard.virtue:
                oCard.addVirtue(oVirt)

    def _add_creeds(self, oCard, sCreed):
        """Add creeds to the card."""
        sCreed = self.oWhiteSp.sub(' ', sCreed).strip()

        if sCreed == '-none-' or sCreed == '':
            return

        for sVal in sCreed.split('/'):
            oCreed = self._oMaker.make_creed(sVal.strip())
            if oCreed not in oCard.creed:
                oCard.addCreed(oCreed)

    def _add_clans(self, oCard, sClan):
        """Add clans to the card."""
        sClan = self.oWhiteSp.sub(' ', sClan).strip()

        if sClan == '-none-' or sClan == '':
            return

        for sVal in sClan.split('/'):
            oClan = self._oMaker.make_clan(sVal.strip())
            if oClan not in oCard.clan:
                oCard.addClan(oClan)

    def _add_cost(self, oCard, sCost):
        """Add the cost to the card, replace 'X' with -1."""
        sCost = self.oWhiteSp.sub(' ', sCost).strip()
        sAmnt, sType = sCost.split()

        if sAmnt.lower() == 'x':
            iCost = -1
        else:
            iCost = int(sAmnt, 10)

        oCard.cost = iCost
        oCard.costtype = str(sType.lower())  # make str non-unicode

    def _add_group(self, oCard, sGroup):
        """Add the group to the card. Replace '*' with -1."""
        sGroup = self.oWhiteSp.sub(' ', sGroup).strip()

        if sGroup == '*' or sGroup.lower() == 'any':
            iGroup = -1
        else:
            iGroup = int(sGroup, 10)

        oCard.group = iGroup

    def _add_life(self, oCard, sLife):
        """Add the life to the card."""
        sLife = self.oWhiteSp.sub(' ', sLife).strip()
        aLife = sLife.split()
        # pylint: disable-msg=W0704
        # ignoring the error is the right thing here
        try:
            oCard.life = int(aLife[0], 10)
        except ValueError:
            pass

    def _get_level(self, sLevel):
        """Normalised the level string."""
        return self.oWhiteSp.sub(' ', sLevel).strip().lower()

    def _add_level(self, oCard, sLevel):
        """Add the correct string for the level to the card."""
        oCard.level = str(self._get_level(sLevel))  # make str non-unicode

    def _add_level_to_name(self, sName, sLevel):
        """Add level info to the vampire name."""
        return sName.strip() + " (%s)" % self._get_level(sLevel).capitalize()

    def _add_capacity(self, oCard, sCap):
        """Add the capacity to the card."""
        sCap = self.oWhiteSp.sub(' ', sCap).strip()
        aCap = sCap.split()
        # pylint: disable-msg=W0704
        # ignoring the error is the right thing here
        try:
            oCard.capacity = int(aCap[0], 10)
        except ValueError:
            pass

    def _add_card_type(self, oCard, sTypes):
        """Add the card type info to the card."""
        for sVal in sTypes.split('/'):
            oType = self._oMaker.make_card_type(sVal.strip())
            if oType not in oCard.cardtype:
                oCard.addCardType(oType)

    def _add_title(self, oCard, sTitle):
        """Add the title to the card."""
        oTitle = self._oMaker.make_title(sTitle)
        if oTitle not in oCard.title:
            oCard.addTitle(oTitle)

    def _add_sect(self, oCard, sSect):
        """Add the sect to the card."""
        oSect = self._oMaker.make_sect(sSect)
        if oSect not in oCard.sect:
            oCard.addSect(oSect)

    def _add_keyword(self, oCard, sKeyword):
        """Add the keyword to the card."""
        oKeyword = self._oMaker.make_keyword(sKeyword)
        if oKeyword not in oCard.keywords:
            oCard.addKeyword(oKeyword)

    def _add_artists(self, oCard, sArtists):
        """Add the artist to the card."""
        for sArtist in self.oArtistSp.split(sArtists):
            oArtist = self._oMaker.make_artist(sArtist.strip())
            if oArtist not in oCard.artists:
                oCard.addArtist(oArtist)

    def _add_physical_cards(self, oCard):
        """Create a physical card for each expansion."""
        self._oMaker.make_physical_card(oCard, None)
        for oExp in set([oRarity.expansion for oRarity in oCard.rarity]):
            self._oMaker.make_physical_card(oCard, oExp)

    def save(self):
        # pylint: disable-msg=R0912
        # Need to consider all cases, so many branches
        """Commit the card to the database.

           This fills in the needed fields and creates entries in the join
           tables as needed.
           """
        if not self.has_key('name'):
            return

        if self.has_key('level'):
            self['name'] = self._add_level_to_name(self['name'], self['level'])

        oCard = self._make_card(self['name'])

        self.oLogger.info('Card: %s', self['name'])

        if self.has_key('text'):
            self._parse_text(oCard)

        if self.has_key('group'):
            self._add_group(oCard, self['group'])

        if self.has_key('capacity'):
            self._add_capacity(oCard, self['capacity'])

        if self.has_key('cost'):
            self._add_cost(oCard, self['cost'])

        if self.has_key('life'):
            self._add_life(oCard, self['life'])

        if self.has_key('level'):
            self._add_level(oCard, self['level'])
            # Also add a keyword for this
            self._add_keyword(oCard, oCard.level)

        if self.has_key('expansion'):
            self._add_expansions(oCard, self['expansion'])

        if self.has_key('discipline'):
            self._add_disciplines(oCard, self['discipline'])

        if self.has_key('virtue'):
            self._add_virtues(oCard, self['virtue'])

        if self.has_key('clan'):
            self._add_clans(oCard, self['clan'])

        if self.has_key('creed'):
            self._add_creeds(oCard, self['creed'])

        if self.has_key('cardtype'):
            self._add_card_type(oCard, self['cardtype'])

        if self.has_key('burn option'):
            self._add_keyword(oCard, "burn option")

        if self.has_key('title'):
            self._add_title(oCard, self['title'])

        if self.has_key('sect'):
            self._add_sect(oCard, self['sect'])

        if self.has_key('artist'):
            self._add_artists(oCard, self['artist'])

        if self.has_key('text'):
            oCard.text = self['text'].replace('\r', '')

        self._add_blood_shadowed_court(oCard)

        self._add_physical_cards(oCard)

        oCard.syncUpdate()
        # FIXME: Pass back any error confitions? Missing text, etc.


# Parsing helper functions
def fix_clarification_markers(sLine):
    """Standardise the clarification markers from the text"""
    for sMarker in ['={', '-{']:
        sLine = sLine.replace(sMarker, '{')
    for sMarker in ['}=', '}-']:
        sLine = sLine.replace(sMarker, '}')
    return sLine


# State Classes
class WaitingForCardName(LogStateWithInfo):
    """State when we are not in a card."""

    def transition(self, sLine, _dAttr):
        """Transition to PotentialCard if needed."""
        if sLine.startswith('Name:'):
            self.flush()
            sData = sLine.split(':', 1)[1]
            # CSV style names sometimes creep into the cardlist.txt,
            # so ensure we normalise them consistently
            self._dInfo['name'] = csv_to_canonical(sData.strip())
            return InExpansion(self._dInfo, self.oLogger)
        elif sLine.strip():
            if self._dInfo.has_key('name') and not self._dInfo.has_key('text'):
                # We've it a blank line in the middle of a card, so bounce
                # back to InCard stuff
                oCard = InCard(self._dInfo, self.oLogger)
                return oCard.transition(sLine, None)
        return self

    def flush(self):
        """Save any existing card and clear out dInfo"""
        if self._dInfo.has_key('name'):
            # Ensure we've saved existing card
            self._dInfo.save()
        self._dInfo = CardDict(self.oLogger)


class InCard(LogStateWithInfo):
    """State when we are in a card, waiting for the card text."""

    # These look like they start a key: value pair, but actually start
    # card text

    aTextTags = [
            'master',
            'camarilla',
            'sabbat',
            'laibon',
            'independent',
            'strike',
            'weapon',
            ]

    def transition(self, sLine, _dAttr):
        oCardText = None
        if ':' in sLine:
            sTag = fix_clarification_markers(sLine.split(':', 1)[0].lower())
            if sTag in self.aTextTags or ' ' in sTag or '{' in sTag:
                oCardText = InCardText(self._dInfo, self.oLogger)
            else:
                sData = fix_clarification_markers(sLine.split(':', 1)[1])
                # We don't want clarification markers here
                sData = sData.replace('{', '').replace('}', '')
                self._dInfo[sTag] = sData.strip()
        elif not sLine.strip():
            # Empty line, so probably end of the card
            return WaitingForCardName(self._dInfo, self.oLogger)
        else:
            if sLine.strip().lower() == 'burn option':
                # Annoying special case
                self._dInfo['burn option'] = None
            else:
                oCardText = InCardText(self._dInfo, self.oLogger)
        if oCardText:
            # Hand over the line to the card text parser
            return oCardText.transition(sLine, None)
        else:
            return self


class InExpansion(LogStateWithInfo):
    """In the expansions section."""

    def transition(self, sLine, _dAttr):
        """Transition back to InCard if needed."""
        if sLine.startswith('[') and sLine.strip().endswith(']'):
            self._dInfo['expansion'] = sLine.strip()
            return InCard(self._dInfo, self.oLogger)
        else:
            return self


class InCardText(LogStateWithInfo):
    """In the card text section."""

    def transition(self, sLine, _dAttr):
        """Transition back to InCard if needed."""
        if sLine.startswith('Artist:') or not sLine.strip():
            self._dInfo['text'] = fix_clarification_markers(
                    self._dInfo['text'].strip())
            oInCard = InCard(self._dInfo, self.oLogger)
            return oInCard.transition(sLine, None)
        else:
            if not self._dInfo.has_key('text'):
                self._dInfo['text'] = sLine.strip() + '\n'
            else:
                # Since we're building this up a line at a time, we add a
                # trailing space, which we strip before we exit this parser
                self._dInfo['text'] += sLine.strip() + ' '
            return self


# Parser
class WhiteWolfTextParser(object):
    """Actual Parser for the WW cardlist text file(s)."""

    def __init__(self, oLogHandler):
        self.oLogger = Logger('White wolf card parser')
        if oLogHandler is not None:
            self.oLogger.addHandler(oLogHandler)
        self._oState = None
        self.reset()

    def reset(self):
        """Reset the parser"""
        self._oState = WaitingForCardName({}, self.oLogger)

    def parse(self, fIn):
        """Feed lines to the state machine"""
        for sLine in fIn:
            self.feed(sLine)
        # Ensure we flush any open card text states
        self.feed('')
        if hasattr(self._oState, 'flush'):
            self._oState.flush()
        else:
            raise IOError('Failed to parse card list - '
                    'unexpected state at end of file.\n'
                    'Card list probably truncated.')

    def feed(self, sLine):
        """Feed the line to the current state"""
        # Strip BOM from line start
        sLine = sLine.decode('utf8').lstrip(u'\ufeff')
        self._oState = self._oState.transition(sLine, None)



# ZipFileWrapper.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2007 Simon Cross <hodgestar@gmail.com>,
# Copyright 2007 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details
# Class and functions to manage zip file handling for Sutekh
# Split off from SutekhUtility.py and refactored, April 2007  - NM

"""Provide a ZipFile class which wraps the functionlity from zipfile
   Sutekh needs."""

import zipfile
import datetime
from StringIO import StringIO
from logging import Logger
from sqlobject import sqlhub, SQLObjectNotFound
from sutekh.core.SutekhObjects import PhysicalCardSet, PHYSICAL_SET_LIST
from sutekh.core.CardLookup import DEFAULT_LOOKUP
from sutekh.core.CardSetHolder import CachedCardSetHolder, CardSetWrapper
from sutekh.SutekhUtility import refresh_tables
from sutekh.io.PhysicalCardParser import PhysicalCardParser
from sutekh.io.PhysicalCardSetParser import PhysicalCardSetParser
from sutekh.io.AbstractCardSetParser import AbstractCardSetParser
from sutekh.io.PhysicalCardSetWriter import PhysicalCardSetWriter
from sutekh.io.IdentifyXMLFile import IdentifyXMLFile


def _parse_string(oParser, sIn, oHolder):
    """Utitlity function for reading zip files.

       Allows oParser.parse to be called on a string."""
    oFile = StringIO(sIn)
    oParser.parse(oFile, oHolder)


class ZipFileWrapper(object):
    """The zip file wrapper.

       This provides useful functions for dumping + extracting the
       database to / form a zipfile"""
    def __init__(self, sZipFileName):
        self.sZipFileName = sZipFileName
        self.oZip = None
        self._aWarnings = []

    def __open_zip_for_write(self):
        """Open zip file to be written"""
        self.oZip = zipfile.ZipFile(self.sZipFileName, 'w')

    def __open_zip_for_read(self):
        """Open zip file to be read"""
        self.oZip = zipfile.ZipFile(self.sZipFileName, 'r')

    def __close_zip(self):
        """Close the zip file and clean up"""
        self.oZip.close()
        self.oZip = None

    def write_pcs_list_to_zip(self, aPCSList, oLogger):
        """Write the given list of card sets to the zip file"""
        bClose = False
        tTime = datetime.datetime.now().timetuple()
        if self.oZip is None:
            self.__open_zip_for_write()
            bClose = True
        aList = []
        for oPCSet in aPCSList:
            sZName = oPCSet.name
            oWriter = PhysicalCardSetWriter()
            oHolder = CardSetWrapper(oPCSet)
            oFile = StringIO()
            oWriter.write(oFile, oHolder)
            oString = oFile.getvalue()
            oFile.close()
            sZName = sZName.replace(" ", "_")
            sZName = sZName.replace("/", "_")
            sZipName = '%s.xml' % sZName
            sZipName = sZipName.encode('ascii', 'xmlcharrefreplace')
            aList.append(sZipName)
            # ZipInfo will just use the 1st 6 fields in tTime
            oInfoObj = zipfile.ZipInfo(sZipName, tTime)
            # Set permissions on the created file - see issue 3394 on the
            # python bugtracker. Docs say this is safe on all platforms
            oInfoObj.external_attr = 0600 << 16L
            oInfoObj.compress_type = zipfile.ZIP_DEFLATED
            self.oZip.writestr(oInfoObj, oString)
            oLogger.info('PCS: %s written', oPCSet.name)
        if bClose:
            self.__close_zip()
        return aList

    def do_restore_from_zip(self, oCardLookup=DEFAULT_LOOKUP,
            oLogHandler=None):
        """Recover data from the zip file"""
        self._aWarnings = []
        bTablesRefreshed = False
        bOldStyle = False
        self.__open_zip_for_read()
        oLogger = Logger('Restore zip file')
        if oLogHandler is not None:
            oLogger.addHandler(oLogHandler)
            if hasattr(oLogHandler, 'set_total'):
                oLogHandler.set_total(len(self.oZip.infolist()))
        # We do this so we can accomodate user created zipfiles,
        # that don't nessecarily have the ordering we want
        oIdParser = IdentifyXMLFile()
        # check that the zip file contains at least 1 PCS or the old
        # PhysicalCard list
        for oItem in self.oZip.infolist():
            oData = self.oZip.read(oItem.filename)
            _parse_string(oIdParser, oData, None)
            if (oIdParser.type == 'PhysicalCard' or \
                    oIdParser.type == 'PhysicalCardSet') and not \
                    bTablesRefreshed:
                # We delete the Physical Card Sets
                # Since this is restoring the contents of a zip file,
                # hopefully this is safe to do
                # if we fail, the database will be in an inconsitent state,
                # but that's going to be true anyway
                refresh_tables(PHYSICAL_SET_LIST, sqlhub.processConnection)
                bTablesRefreshed = True
            if oIdParser.type == 'PhysicalCard':
                bOldStyle = True
        if not bTablesRefreshed:
            raise IOError("No valid card sets found in the zip file.")
        # We try and restore the old PCS's ensuring parents exist
        dLookupCache = {}
        aToRead = self.oZip.infolist()
        while len(aToRead) > 0:
            aToRead = self.read_items(aToRead, oCardLookup, oLogger, bOldStyle,
                    dLookupCache)
        self.__close_zip()

    # pylint: disable-msg=R0913
    # we may need all these arguments
    def read_items(self, aList, oCardLookup, oLogger, bOldStyle, dLookupCache):
        """Read a list of CardSet items from the card list, reaturning
           a list of those that couldn't be read because their parents
           weren't read first"""
        aToRead = []
        oIdParser = IdentifyXMLFile()
        for oItem in aList:
            bReparent = False
            oData = self.oZip.read(oItem.filename)
            _parse_string(oIdParser, oData, None)
            if oIdParser.type == 'PhysicalCardSet':
                # We check whether the parent has been read already
                if bOldStyle:
                    # We need to reparent this card set
                    try:
                        # card set holder will handle setting the parent
                        PhysicalCardSet.selectBy(
                                name='My Collection').getOne()
                        bReparent = True
                        oParser = PhysicalCardSetParser()
                    except SQLObjectNotFound:
                        # Card Collection not there yet, so delay
                        aToRead.append(oItem)
                        continue
                elif oIdParser.parent_exists:
                    oParser = PhysicalCardSetParser()
                else:
                    aToRead.append(oItem)
                    continue
            elif oIdParser.type == 'AbstractCardSet':
                oParser = AbstractCardSetParser()
            elif oIdParser.type == 'PhysicalCard':
                oParser = PhysicalCardParser()
            else:
                continue
            oHolder = CachedCardSetHolder()
            _parse_string(oParser, oData, oHolder)
            if bReparent:
                # pylint: disable-msg=E1103
                # SQLObject confuses pylint
                oHolder.parent = 'My Collection'
            oHolder.create_pcs(oCardLookup, dLookupCache)
            self._aWarnings.extend(oHolder.get_warnings())
            oLogger.info('%s %s read', oIdParser.type, oItem.filename)
        if len(aToRead) == len(aList):
            # We were unable to read any items this loop, so we fail
            raise IOError('Card sets with unstatisfiable parents %s' %
                    ','.join([x.filename for x in aToRead]))
        return aToRead

    # pylint: enable-msg=R0913

    def do_dump_all_to_zip(self, oLogHandler=None):
        """Dump all the database contents to the zip file"""
        aPhysicalCardSets = PhysicalCardSet.select()
        return self.do_dump_list_to_zip(aPhysicalCardSets, oLogHandler)

    def do_dump_list_to_zip(self, aCSList, oLogHandler=None):
        """Handle dumping a list of cards to the zip file with log fiddling"""
        self.__open_zip_for_write()
        oLogger = Logger('Write zip file')
        if oLogHandler is not None:
            oLogger.addHandler(oLogHandler)
            if hasattr(oLogHandler, 'set_total'):
                if hasattr(aCSList, 'count'):
                    # Handle case we have a select result list
                    iTotal = aCSList.count()
                    oLogHandler.set_total(iTotal)
                else:
                    oLogHandler.set_total(len(aCSList))
        aPCSList = self.write_pcs_list_to_zip(aCSList, oLogger)
        self.__close_zip()
        return aPCSList

    def get_all_entries(self):
        """Return the list of card sets in the zip file"""
        self.__open_zip_for_read()
        dCardSets = {}
        oIdParser = IdentifyXMLFile()
        for oItem in self.oZip.infolist():
            oData = self.oZip.read(oItem.filename)
            _parse_string(oIdParser, oData, None)
            if oIdParser.type == 'PhysicalCardSet':
                dCardSets[oIdParser.name] = (oItem.filename,
                        oIdParser.parent_exists, oIdParser.parent)
        self.__close_zip()
        return dCardSets

    def read_single_card_set(self, sFilename):
        """Read a single card set into a card set holder."""
        self.__open_zip_for_read()
        oIdParser = IdentifyXMLFile()
        oData = self.oZip.read(sFilename)
        _parse_string(oIdParser, oData, None)
        oHolder = None
        if oIdParser.type == 'PhysicalCardSet':
            oParser = PhysicalCardSetParser()
            oHolder = CachedCardSetHolder()
            _parse_string(oParser, oData, oHolder)
        self.__close_zip()
        return oHolder

    def get_warnings(self):
        """Get any warnings from the process"""
        return self._aWarnings



# CellRendererSutekhButton.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Pixbuf Button CellRenderer
# Copyright 2006 Neil Muller <drnlmuller+sutekh@gmail.com>
# Copyright 2006 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Render a icon in a TreeView"""

import gtk
import gobject

# This is heavily cribbed from the example in the pygtk FAQ
# (By Nikos Kouremenos)
# limited to the use of icons only ATM
# Allows creation of a cell containing a icon pixbuf,
# and returns a "clicked" signal when activated
# To be generically useful, should be extended to abitary pixmaps,
# but that is currently not a priority


# pylint: disable-msg=R0904
# gtk widget, so we must have a lot of public methods
class CellRendererSutekhButton(gtk.GenericCellRenderer):
    """Render a icon in a cell in a TreeView.

       Used to render the arrows for incrementing and decrementing cards
       in the CardListView's
       """
    # Register a showicon property - used to control wether the icon
    # is visible or not
    __gproperties__ = {
            'showicon': (gobject.TYPE_BOOLEAN, 'showicon property',
                'whether to show the icon', 0, gobject.PARAM_READWRITE),
            }

    def __init__(self, bShowIcon=False):
        super(CellRendererSutekhButton, self).__init__()
        self.__gobject_init__()
        self.oPixbuf = None
        self.set_property("mode", gtk.CELL_RENDERER_MODE_ACTIVATABLE)
        self.bShowIcon = bShowIcon
        self.bClicked = False
        self.oClickedBackgroundArea = None

    def load_icon(self, sName, oWidget):
        """Load the icon specified in name"""
        self.oPixbuf = oWidget.render_icon(sName, gtk.ICON_SIZE_MENU)

    def do_get_property(self, oProp):
        """Allow reading the showicon property"""
        if oProp.name == 'showicon':
            return self.bShowIcon
        else:
            raise AttributeError, 'unknown property %s' % oProp.name

    def do_set_property(self, oProp, oValue):
        """Allow setting the showicon property"""
        if oProp.name == 'showicon':
            self.bShowIcon = oValue
        else:
            raise AttributeError, 'unknown property %s' % oProp.name

    def on_get_size(self, _oWidget, oCellArea):
        """Handle get_size requests"""
        if self.oPixbuf is None:
            return 0, 0, 0, 0
        iPixbufWidth = self.oPixbuf.get_width()
        iPixbufHeight = self.oPixbuf.get_height()
        fCalcWidth = self.get_property("xpad") * 2 + iPixbufWidth
        fCalcHeight = self.get_property("ypad") * 2 + iPixbufHeight
        iXOffset = 0
        iYOffset = 0
        if oCellArea is not None and iPixbufWidth > 0 and iPixbufHeight > 0:
            iXOffset = int(self.get_property("xalign") * (oCellArea.width - \
                fCalcWidth - self.get_property("xpad")))
            iYOffset = int(self.get_property("yalign") * (oCellArea.height - \
                fCalcHeight - self.get_property("ypad")))
        # gtk want's ints here
        return iXOffset, iYOffset, int(fCalcWidth), int(fCalcHeight)

    # pylint: disable-msg=R0913
    # R0913 - number of parameters needed by function signature
    def on_activate(self, _oEvent, _oWidget, oPath, oBackgroundArea,
            _oCellArea, _iFlags):
        """Activate signal received from the TreeView"""
        # Note that we need to offset button
        self.bClicked = True
        self.oClickedBackgroundArea = oBackgroundArea
        # clicked callback should be called
        self.emit('clicked', oPath)
        return True

    # pylint: disable-msg=R0913
    # R0913 - number of parameters needed by function signature
    def on_render(self, oWindow, oWidget, oBackgroundArea,
            oCellArea, oExposeArea, _iFlags):
        """Render the icon for the button"""
        bDrawOffset = False
        # Need to ensure that self.bClicked is unset before any early return
        if self.bClicked:
            # Offset when button is clicked
            if oBackgroundArea.x == self.oClickedBackgroundArea.x and \
                    oBackgroundArea.y == self.oClickedBackgroundArea.y:
                # Rendering the same area as was clicked, so we need to offset
                bDrawOffset = True
                # reset clicked state
                self.bClicked = False
        if self.oPixbuf is None:
            return None
        if not self.bShowIcon:
            # Draw nothing
            return None
        oPixRect = gtk.gdk.Rectangle()
        oPixRect.x, oPixRect.y, oPixRect.width, oPixRect.height = \
            self.on_get_size(oWidget, oCellArea)

        oPixRect.x += oCellArea.x
        oPixRect.y += oCellArea.y
        # xpad, ypad are floats, but gtk.gdk.Rectangle needs int's
        oPixRect.width -= int(2 * self.get_property("xpad"))
        oPixRect.height -= int(2 * self.get_property("ypad"))

        if bDrawOffset:
            oPixRect.x += 1
            oPixRect.y += 1
            # We add a timeout to force a redraw to unbump the button
            gobject.timeout_add(200, self.restore_offset, oWindow,
                    oBackgroundArea)

        oDrawRect = oCellArea.intersect(oPixRect)
        oDrawRect = oExposeArea.intersect(oDrawRect)

        oWindow.draw_pixbuf(oWidget.style.black_gc, self.oPixbuf,
            oDrawRect.x - oPixRect.x, oDrawRect.y - oPixRect.y, oDrawRect.x,
            oDrawRect.y, oDrawRect.width, oDrawRect.height,
            gtk.gdk.RGB_DITHER_NONE, 0, 0)
        return None

    def restore_offset(self, oWindow, oArea):
        """queue a redraw so we restore the button."""
        # Don't unbump if the user has clicked again
        if not self.bClicked:
            # queue a redraw
            oWindow.invalidate_rect(oArea, False)


# HouseKeeping work for CellRendererStukehButton
# Awkward stylistically, but I'm putting it here as it's
# associated with class creation (is global to the class)
# so doesn't belong in the per-instance constructor - NM
# Register class, needed to make clicked signal go to right class
# (otherwise it gets associated with GenericCellRenderer, which is
# not desireable)
gobject.type_register(CellRendererSutekhButton)
# Setup clicked signal -- can also be done using the __gsignals__
# dict in the class, but I couldn't find good documentation for
# that approach.
gobject.signal_new("clicked", CellRendererSutekhButton,
    gobject.SIGNAL_RUN_FIRST | gobject.SIGNAL_ACTION,
    gobject.TYPE_NONE,
    (gobject.TYPE_STRING,))
# the callback is called as callback (self, oPath)



# MultiSelectComboBox.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Generic multiselect combobox for use in FilterEditor (and elsewhere)"""

import gtk
import sys
from sutekh.gui.AutoScrolledWindow import AutoScrolledWindow
from sutekh.gui.ScrolledList import ScrolledListView


def mouse_in_button(oButton):
    """Check if mouse pointer is inside the button"""
    (iXPos, iYPos) = oButton.get_pointer()  # mouse pos relative to button
    oButtonGeom = oButton.allocation
    return (iXPos >= 0) and (iYPos >= 0) and \
            (iXPos < oButtonGeom.width) and (iYPos < oButtonGeom.height)


class MultiSelectComboBox(gtk.HBox):
    # pylint: disable-msg=R0904
    # gtk.Widget, so many public methods
    """Implementation of a multiselect combo box widget."""

    def __init__(self, oParentWin):
        super(MultiSelectComboBox, self).__init__()

        self._oButton = gtk.Button(" - ")
        self._oButton.connect('clicked', self.__show_list)
        self.pack_start(self._oButton)

        self._oTreeView = ScrolledListView(" ... ")
        self._oTreeView.get_selection().set_mode(gtk.SELECTION_MULTIPLE)

        oScrolled = AutoScrolledWindow(self._oTreeView)
        self._aOldSelection = []

        self._oDialog = gtk.Dialog("Select ...", None,
                gtk.DIALOG_MODAL | gtk.DIALOG_NO_SEPARATOR |
                gtk.DIALOG_DESTROY_WITH_PARENT)
        self._oDialog.set_decorated(False)
        # pylint: disable-msg=E1101
        # action_area, vbox confuses pylint
        self._oDialog.action_area.set_size_request(-1, 0)
        self._oDialog.vbox.pack_start(oScrolled)
        self._oDialog.connect('key-press-event', self.__hide_on_return)
        # Catch tail of the event queue to handle pressing to close
        self._oDialog.connect('event-after', self.__grab_event)

        self._bInButton = False
        self._oParentWin = oParentWin

    def __grab_event(self, _oWidget, oEvent):
        """Hook into the event-after chain, so we can check if any uncaught
           events refer to the original button."""
        # This is a bit convuluted, but seems the best
        # we can do.
        # If the popup dialog isn't modal, it can't reliably receive focus, and
        # there are undesireable interactions with the default keybindings on
        # the parent dialog.
        # If it's modal, the button can't catch any events. Thus we fake
        # limited non-modality here by testing for each event of interest
        # ourselves. This will all go away if the popup is redone as a
        # widget, rather than as a tweaked dialog window
        if oEvent.type == gtk.gdk.BUTTON_PRESS or (
                sys.platform.startswith("win") and
                oEvent.type == gtk.gdk.BUTTON_RELEASE):
            # Mouse clicked
            if oEvent.button == 1:
                if mouse_in_button(self._oButton):
                    # Left button
                    self.__hide_list()
                # Ignore other buttons
                # Should right button act the same as escape?
        elif oEvent.type == gtk.gdk.ENTER_NOTIFY:
            # Mouse has entered the button, so mark as active
            if mouse_in_button(self._oButton):
                self._bInButton = True
                self._oButton.set_state(gtk.STATE_PRELIGHT)
        elif oEvent.type == gtk.gdk.LEAVE_NOTIFY and self._bInButton:
            # Leave the button, so unhighlight
            self._bInButton = False
            self._oButton.set_state(gtk.STATE_NORMAL)
        # always propogate events onward, which should be completely
        # safe, since we're in event-after
        return False

    def __show_list(self, _oButton):
        """Drop down the list of possible selections."""
        self._aOldSelection = self.get_selection()

        oParent = self.get_parent_window()

        tWinPos = oParent.get_origin()
        # Need coordinates relative to root window
        # pylint: disable-msg=E1101
        # allocation confuses pylint
        tButtonPos = (self._oButton.allocation.x, self._oButton.allocation.y)
        tShift = (5, self._oButton.allocation.height)

        tDialogPos = (tWinPos[0] + tButtonPos[0] + tShift[0],
                tWinPos[1] + tButtonPos[1] + tShift[1])

        self._oDialog.set_keep_above(True)  # Keep this above the dialog
        self._oDialog.set_transient_for(self._oParentWin)
        self._oDialog.show_all()
        # WM behaviour means that move is unlikely to work before _oDialog
        # is shown
        self._oDialog.move(tDialogPos[0], tDialogPos[1])
        self._bInButton = False

    def __hide_on_return(self, _oWidget, oEvent):
        """Hide the list when return or escape is pressed."""
        if oEvent.type is gtk.gdk.KEY_PRESS:
            sKeyName = gtk.gdk.keyval_name(oEvent.keyval)
            if sKeyName in ('Return', 'Escape'):
                if sKeyName == 'Escape':
                    self.set_selected_rows(self._aOldSelection)
                self.__hide_list()
                return True  # event handled
        return False  # process further

    def __hide_list(self):
        """Hide the list of options"""
        self._oDialog.hide_all()
        self.__update_button_text()

    def __update_button_text(self):
        """Update the text to reflect the selected items."""
        aSelection = self.get_selection()
        if aSelection:
            self._oButton.set_label(", ".join(aSelection))
        else:
            self._oButton.set_label(" - ")

    def fill_list(self, aVals):
        """Fill the list store with the given values"""
        self._oTreeView.store.fill_list(aVals)

    def set_list_size(self, iWidth, iHeight):
        """Set size of the drop-down list"""
        self._oDialog.set_size_request(iWidth, iHeight)

    def set_sensitive(self, bValue):
        """Control the sensitivity of the button"""
        self._oButton.set_sensitive(bValue)

    def get_selection(self):
        """Return a list of the selected elements of the list"""
        return self._oTreeView.get_selected_data()

    def set_selected_rows(self, aRowsToSelect):
        """Set the selected rows in the drop-down to aRowsToSelect"""
        self._oTreeView.set_selected_rows(aRowsToSelect)



# test_ARDBXMLDeckParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test reading a card set from an ARDB XML deck file"""

import unittest
from sutekh.tests.TestCore import SutekhTest
from sutekh.io.ARDBXMLDeckParser import ARDBXMLDeckParser
from sutekh.tests.io.test_AbstractCardSetParser import ACS_EXAMPLE_1

ARDB_DECK_EXAMPLE_1 = """
<deck generator="Sutekh [ Test ]" fromatVersion="val">
    <name>Test Deck</name>
    <author>Anon Y Mous</author>
    <description>Simple test deck.

    http://www.example.url/in/description</description>
    <date>2008-09-01</date>
    <crypt size="3" min="1" max="2" avg="1.1">
        <vampire databaseID="1" count="2">
           <name>Test Vamp 1</name>
           <adv></adv>
           <set>CE</set>
           <capacity></capacity>
           <text></text>
           <group />
           <disciplines></disciplines>
        </vampire>
        <vampire databaseID="2" count="1">
           <name>Test Vamp 2</name>
           <adv></adv>
           <set>SW</set>
           <capacity></capacity>
           <text></text>
           <group></group>
           <disciplines></disciplines>
        </vampire>
        <vampire databaseID="2" count="1">
           <name>Test Vamp 2</name>
           <adv>(Advanced)</adv>
           <set>Promo20051001</set>
           <capacity></capacity>
           <text></text>
           <group></group>
           <disciplines></disciplines>
        </vampire>
    </crypt>
    <library size="17">
       <card databaseID="3" count="4">
          <name>Test Card 1</name>
          <set>Sabbat</set>
          <type></type>
          <cost></cost>
          <requirement></requirement>
          <text />
       </card>
       <card databaseID="3" count="2">
          <name>Test Card 2</name>
          <set>BH</set>
          <type></type>
          <cost></cost>
          <requirement />
          <text></text>
       </card>
       <card databaseID="3" count="12">
          <name>Test Card 3</name>
          <set>BH</set>
          <type></type>
          <cost></cost>
          <requirement></requirement>
          <text></text>
       </card>
       <card databaseID="3" count="1">
          <name>Test Card 4</name>
          <set></set>
          <type></type>
          <cost></cost>
          <requirement></requirement>
          <text></text>
       </card>
       <card databaseID="50" count="1">
          <name>Test Card 5, The</name>
          <set></set>
          <type></type>
          <cost></cost>
          <requirement></requirement>
          <text></text>
       </card>
    </library>
</deck>
"""


class ArdbXMLDeckParserTests(SutekhTest):
    """class for the ARDB XML deck file parser tests"""

    # ARDB produces tag pairs for empty elements, we produce minimal
    # tags (<set></set> vs <set />, so we have both in the test data

    def test_basic(self):
        """Run the input test."""
        oHolder = self._make_holder_from_string(ARDBXMLDeckParser(),
                ARDB_DECK_EXAMPLE_1)

        self.assertEqual(oHolder.name, "Test Deck")
        self.assertEqual(oHolder.author, "Anon Y Mous")
        self.failUnless(oHolder.comment.startswith("Simple test deck."))
        self.failUnless(oHolder.comment.endswith("in/description"))

        aCards = oHolder.get_cards_exps()

        self.assertEqual(len(aCards), 8)
        self.failUnless((("Test Vamp 1", "CE"), 2) in aCards)
        self.failUnless((("Test Vamp 2", "SW"), 1) in aCards)
        self.failUnless((("Test Vamp 2 (Advanced)", "Promo-20051001"), 1)
                in aCards)
        self.failUnless((("Test Card 1", "Sabbat"), 4) in aCards)
        self.failUnless((("Test Card 2", "BH"), 2) in aCards)
        self.failUnless((("Test Card 3", "BH"), 12) in aCards)
        self.failUnless((("Test Card 4", None), 1) in aCards)
        self.failUnless((("The Test Card 5", None), 1) in aCards)

        oParser = ARDBXMLDeckParser()
        self.assertRaises(IOError, self._make_holder_from_string, oParser,
                ACS_EXAMPLE_1)
        self.assertRaises(IOError, self._make_holder_from_string, oParser,
                'random stuff')


if __name__ == "__main__":
    unittest.main()



# CardSetManagementMenu.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Menu for the CardSet View's
# Copyright 2006 Neil Muller <drnlmuller+sutekh@gmail.com>
# Copyright 2006 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""Menu for the card set list"""

import gtk
from sutekh.gui.FilteredViewMenu import FilteredViewMenu
from sutekh.gui.FrameProfileEditor import FrameProfileEditor
from sutekh.gui.ConfigFile import ConfigFileListener, CARDSET_LIST


class CardSetManagementMenu(FilteredViewMenu, ConfigFileListener):
    """Card Set List Management menu.
       """
    # pylint: disable-msg=R0904
    # gtk.Widget, so many public methods
    def __init__(self, oFrame, oWindow, oController):
        super(CardSetManagementMenu, self).__init__(oFrame, oWindow,
                oController)
        self.__sName = 'Card Set List'
        self.__sSetTypeName = 'Card Set'
        self._oController = oController
        self._oCardSetProfileMenu = None
        self.__create_actions_menu()
        self.create_edit_menu()
        self.create_filter_menu()
        self.add_plugins_to_menus(self._oFrame)

    # pylint: disable-msg=W0201
    # called from __init__, so OK
    def __create_actions_menu(self):
        """Add the Actions Menu"""
        oMenu = self.create_submenu(self, "_Actions")
        self.create_menu_item('Create New Card Set', oMenu,
                self._oController.create_new_card_set)
        self.create_menu_item('Edit Card Set Properties', oMenu,
                self._oController.edit_card_set_properties)
        self.create_menu_item('Mark/UnMark Card Set as in use', oMenu,
                    self._oController.toggle_in_use_flag)
        self.create_menu_item('Delete selected Card Set', oMenu,
                self._oController.delete_card_set, 'Delete')
        oMenu.add(gtk.SeparatorMenuItem())
        self.add_common_actions(oMenu)

    # pylint: enable-msg=W0201

    def create_edit_menu(self):
        """Create the edit menu and populate it"""
        oMenu = self.create_submenu(self, "_Edit")

        self.create_menu_item('Edit _Profiles', oMenu, self._edit_profiles)

        sProfile = self._oMainWindow.config_file.get_profile(CARDSET_LIST,
                CARDSET_LIST)
        self._oCardSetProfileMenu = self._create_profile_menu(oMenu,
            "CardSet List Profile", CARDSET_LIST,
            self._select_cardset_list_profile, sProfile)

        self._oMainWindow.config_file.add_listener(self)

        self.add_edit_menu_actions(oMenu)

    def cleanup(self):
        """Remove the menu listener"""
        self._oMainWindow.config_file.remove_listener(self)

    def _edit_profiles(self, _oWidget):
        """Open an options profiles editing dialog."""
        oDlg = FrameProfileEditor(self._oMainWindow,
            self._oMainWindow.config_file, CARDSET_LIST)
        sCurProfile = self._oMainWindow.config_file.get_profile(CARDSET_LIST,
            CARDSET_LIST)
        oDlg.set_selected_profile(sCurProfile)
        oDlg.run()

        self._fix_profile_menu()

    def _fix_profile_menu(self):
        """Set the profile menu correctly"""
        sProfile = self._oMainWindow.config_file.get_profile(CARDSET_LIST,
                CARDSET_LIST)
        self._update_profile_group(self._oCardSetProfileMenu, CARDSET_LIST,
            self._select_cardset_list_profile, sProfile)

    def _select_cardset_list_profile(self, oRadio, sProfileKey):
        """Callback to change the profile of the current card set."""
        if oRadio.get_active():
            oConfig = self._oMainWindow.config_file
            oConfig.set_profile(CARDSET_LIST, CARDSET_LIST, sProfileKey)

    # Respond to profile changes

    def remove_profile(self, sType, _sProfile):
        """A profile has been removed"""
        if sType == CARDSET_LIST:
            self._fix_profile_menu()

    def profile_option_changed(self, sType, _sProfile, sKey):
        """Update menu if profiles are renamed."""
        if sType == CARDSET_LIST and sKey == 'name':
            self._fix_profile_menu()



# __init__.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2007 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""This is the sutekh.core package.
   """



# test_ConfigFileLegacy.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2010 Simon Cross <hodgestar+sutekh@gmail.com>
# GPL - see COPYING for details

"""Tests the legacy config file support"""

from sutekh.tests.TestCore import SutekhTest
from sutekh.gui.ConfigFileLegacy import ConfigFileLegacy
from sutekh.gui.ConfigFile import CARDSET, CARDSET_LIST, FRAME, WW_CARDLIST


EMPTY_LEGACY_CONFIG = """
"""

BASIC_LEGACY_CONFIG = """
[Pane settings]
pane 4 = 1:0:1:My Collection

[GUI Preferences]
window size = 1429, 826
show zero count cards = no
save on exit = yes
database url = sqlite:///home/user/.sutekh/sutekh.db
postfix name display = yes
save window size = yes
save pane sizes = yes

[Plugin Options]
html export mode = Secret Library
show starters = No
card image path = /home/user/.sutekh/cardimages

[Filters]
test filter = Clan in Malkavian

[Open Panes]
pane 4 = physical_card_set.340:My Collection
pane 5 = physical_card.435:White Wolf Card List
pane 1 = Card Set List:Card Set List
pane 2 = Card Text.209:Card Text
pane 3 = Card Image Frame.V.306:Card Image Frame
"""


class ConfigFileLegacyTests(SutekhTest):
    """Tests for the legacy config file support."""

    def _convert_config(self, sConfigData):
        """Run the config update code"""
        sLegacyConf = self._create_tmp_file(sConfigData)
        sNewConf = self._create_tmp_file()
        oLegacyConf = ConfigFileLegacy(sLegacyConf)
        oConfig = oLegacyConf.convert(sNewConf)
        return oConfig, oLegacyConf

    def _check_global_options(self, oNew, oOld):
        """Check that the global options are converted correctly"""
        for sMethod in ('get_database_uri', 'get_icon_path',
                'get_postfix_the_display', 'get_save_on_exit',
                'get_save_precise_pos', 'get_save_window_size',
                'get_window_size'):
            sNewValue = getattr(oNew, sMethod)()
            sOldValue = getattr(oOld, sMethod)()
            self.assertEqual(sNewValue, sOldValue)

    def test_empty_import(self):
        """Test updating from an empty config file"""
        oConfig, oLegacyConfig = self._convert_config(EMPTY_LEGACY_CONFIG)

        self._check_global_options(oConfig, oLegacyConfig)

        self.assertEqual(oConfig.open_frames(), [
            (1, 'physical_card', 'White Wolf Card List', None, False, False,
                -1),
            (2, 'Card Text', 'Card Text', None, False, False, -1),
            (3, 'Card Set List', 'Card Set List', None, False, False, -1),
            (4, 'physical_card_set', 'My Collection', None, False, False, -1),
        ])

        self.assertEqual(oConfig.get_filter_keys(), [])

        self.assertEqual(oConfig.profiles(CARDSET), [])
        self.assertEqual(oConfig.profiles(CARDSET_LIST), [])
        self.assertEqual(oConfig.profiles(FRAME), [])
        self.assertEqual(oConfig.profiles(WW_CARDLIST), [])

    def test_basic_import(self):
        """Test converting a simple old config file"""
        oConfig, oLegacyConfig = self._convert_config(BASIC_LEGACY_CONFIG)

        self._check_global_options(oConfig, oLegacyConfig)
        self.assertEqual(oConfig.open_frames(), [
            (1, 'Card Set List', 'Card Set List', None, False, False, -1),
            (2, 'Card Text', 'Card Text', None, False, False, 209),
            (3, 'Card Image Frame', 'Card Image Frame', None, True, False,
                306),
            (4, 'physical_card_set', 'My Collection', None, False, False, 340),
            (5, 'physical_card', 'White Wolf Card List', None, False, False,
                435),
        ])

        self.assertEqual(oConfig.get_filter_keys(),
                [("test filter", "Clan in Malkavian")])
        self.assertEqual(oConfig.get_filter_values("test filter"), {})

        self.assertEqual(oConfig.profiles(CARDSET), [])
        self.assertEqual(oConfig.profiles(CARDSET_LIST), [])
        self.assertEqual(oConfig.profiles(FRAME), [])
        self.assertEqual(oConfig.profiles(WW_CARDLIST), [])

        for sPlugin, sKey in [
            ('card image path', 'CardImagePlugin'),
            ('HTML export mode', 'CardSetExportHTML'),
            ('show starters', 'StarterInfoPlugin'),
            ]:
            self.assertEqual(oConfig.get_plugin_key(sPlugin, sKey),
                oLegacyConfig.get_plugin_key(sKey))



# HTMLTextView.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# TextView Object that displays an HTML file
# Copyright 2005, 2006, 2007 Gustavo J. A. M. Carneiro
# Changes for Sutekh, Copyright 2008, 2009, 2010
#           Neil Muller <drnlmuller+sutekh@gmail.com>
# License: GPL - See COPYRIGHT file for details
# Original version downloaded from http://www.gnome.org/~gjc/htmltextview.py
# April 2008
#   Original version released under the GNU Lesser General Public License
#   (version 2 or later) - see
#       http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html

"""A gtk.TextView-based renderer for XHTML-IM, as described in:
   http://www.jabber.org/jeps/jep-0071.html .
   """

import gobject
import pango
import gtk
import re
import warnings
import HTMLParser
from cStringIO import StringIO
from sutekh.gui.SutekhDialog import SutekhDialog
from sutekh.gui.AutoScrolledWindow import AutoScrolledWindow

WHITE_SPACE_REGEX = re.compile("\\s+")

# set for fast inclusion queries, since we hit this list multiple times
HTML_HEADING_TAGS = set(('h1', 'h2', 'h3', 'h4', 'h5', 'h6'))

# don't throw an exception if no screen during import - we'll catch that later
if gtk.gdk.screen_get_default() is None:
    SCREEN_RESOLUTION = 0
else:
    ## pixels = points * SCREEN_RESOLUTION
    SCREEN_RESOLUTION = 0.3514598 * (gtk.gdk.screen_height() /
            float(gtk.gdk.screen_height_mm()))


def _parse_css_color(sColor):
    '''_parse_css_color(css_color) -> gtk.gdk.Color'''
    if sColor.startswith("rgb(") and sColor.endswith(')'):
        iRed, iGreen, iBlue = [int(c) * 257 for c in sColor[4:-1].split(',')]
        return gtk.gdk.Color(iRed, iGreen, iBlue)
    else:
        return gtk.gdk.color_parse(sColor)


class HtmlHandler(HTMLParser.HTMLParser):
    # pylint: disable-msg=R0201, R0902
    # R0201: can't break these into functions
    # R0902: We need to keep a lot of state to handle HTML properly
    """Parse the HTML input and update the gtk.TextView"""
    def __init__(self, oTextView, oStartIter, fLinkLoader):
        HTMLParser.HTMLParser.__init__(self)
        self._oTextBuf = oTextView.get_buffer()
        self._oTextView = oTextView
        self._oIter = oStartIter
        self._sText = ''
        self._aStyles = []  # a gtk.TextTag or None, for each span level
        self._aListCounters = []  # stack (top at head) of list
                                # counters, or None for unordered list
        self._bInTitle = False
        self._dTargets = {}
        self._fLinkLoader = fLinkLoader

    def _parse_style_color(self, oTag, sValue):
        """Convert style value to TextView foreground color"""
        oColor = _parse_css_color(sValue)
        oTag.set_property("foreground-gdk", oColor)

    def _parse_style_background_color(self, oTag, sValue):
        """Convert background value to TextView background color."""
        oColor = _parse_css_color(sValue)
        oTag.set_property("background-gdk", oColor)
        oTag.set_property("paragraph-background-gdk", oColor)

    def _get_current_attributes(self):
        """Get current attributes."""
        aAttrs = self._oTextView.get_default_attributes()
        self._oIter.backward_char()
        self._oIter.get_attributes(aAttrs)
        self._oIter.forward_char()
        return aAttrs

    def _parse_length(self, sValue, bFontRelative, fCallback, *args):
        """Parse/calc length, converting to pixels.

           calls callback(length, *args) when the length is first computed
           or changes
           """
        # pylint: disable-msg=W0142
        # *magic required here
        if sValue.endswith('%'):
            fFrac = float(sValue[:-1]) / 100
            if bFontRelative:
                oAttrs = self._get_current_attributes()
                fFontSize = oAttrs.font.get_size() / pango.SCALE
                fCallback(fFrac * SCREEN_RESOLUTION * fFontSize, *args)
            else:
                ## CSS says "Percentage values: refer to width of the closest
                ##           block-level ancestor"
                ## This is difficult/impossible to implement, so we use
                ## textview width instead; a reasonable approximation..
                oAlloc = self._oTextView.get_allocation()
                self.__parse_length_frac_cb(self._oTextView, oAlloc,
                        fFrac, fCallback, args)
                self._oTextView.connect("size-allocate",
                        self.__parse_length_frac_cb, fFrac,
                        fCallback, args)

        elif sValue.endswith('pt'):  # points
            fCallback(float(sValue[:-2]) * SCREEN_RESOLUTION, *args)
        elif sValue.endswith('em'):  # ems, the height of the element's font
            oAttrs = self._get_current_attributes()
            fFontSize = oAttrs.font.get_size() / pango.SCALE
            fCallback(float(sValue[:-2]) * SCREEN_RESOLUTION * fFontSize,
                    *args)
        elif sValue.endswith('ex'):  # x-height, ~ the height of the letter 'x'
            ## FIXME: figure out how to calculate this correctly
            ##        for now 'em' size is used as approximation
            oAttrs = self._get_current_attributes()
            fFontSize = oAttrs.font.get_size() / pango.SCALE
            fCallback(float(sValue[:-2]) * SCREEN_RESOLUTION * fFontSize,
                    *args)
        elif sValue.endswith('px'):  # pixels
            fCallback(int(sValue[:-2]), *args)
        else:
            warnings.warn("Unable to parse length value '%s'" % sValue)

    @staticmethod
    def __parse_font_size_cb(fLength, oTag):
        """Callback for font size calculations."""
        oTag.set_property("size-points", fLength / SCREEN_RESOLUTION)

    def _parse_style_font_size(self, oTag, sValue):
        """Parse the font size attribute"""
        try:
            iScale = {
                "xx-small": pango.SCALE_XX_SMALL,
                "x-small": pango.SCALE_X_SMALL,
                "small": pango.SCALE_SMALL,
                "medium": pango.SCALE_MEDIUM,
                "large": pango.SCALE_LARGE,
                "x-large": pango.SCALE_X_LARGE,
                "xx-large": pango.SCALE_XX_LARGE,
                } [sValue]
        except KeyError:
            pass
        else:
            oAttrs = self._get_current_attributes()
            oTag.set_property("scale", iScale / oAttrs.font_scale)
            return
        if sValue == 'smaller':
            oTag.set_property("scale", pango.SCALE_SMALL)
            return
        if sValue == 'larger':
            oTag.set_property("scale", pango.SCALE_LARGE)
            return
        self._parse_length(sValue, True, self.__parse_font_size_cb, oTag)

    def _parse_style_font_style(self, oTag, sValue):
        """Parse the font style attribute"""
        try:
            iStyle = {
                "normal": pango.STYLE_NORMAL,
                "italic": pango.STYLE_ITALIC,
                "oblique": pango.STYLE_OBLIQUE,
                } [sValue]
        except KeyError:
            warnings.warn("unknown font-style %s" % sValue)
        else:
            oTag.set_property("style", iStyle)

    @staticmethod
    def __frac_length_tag_cb(fLength, oTag, sPropName):
        """Callback used by _parse_length for handling the margin style
           settings."""
        oTag.set_property(sPropName, fLength)

    def _parse_style_margin_left(self, oTag, sValue):
        """Handle the margin left style attribute."""
        self._parse_length(sValue, False, self.__frac_length_tag_cb, oTag,
                "left-margin")

    def _parse_style_margin_right(self, oTag, sValue):
        """Handle the margin right style attribute."""
        self._parse_length(sValue, False, self.__frac_length_tag_cb, oTag,
                "right-margin")

    def _parse_style_font_weight(self, oTag, sValue):
        """Adjust the font to match the font weight specification."""
        ## missing 'bolder' and 'lighter', but that's not important for us
        try:
            iWeight = {
                '100': pango.WEIGHT_ULTRALIGHT,
                '200': pango.WEIGHT_ULTRALIGHT,
                '300': pango.WEIGHT_LIGHT,
                '400': pango.WEIGHT_NORMAL,
                '500': pango.WEIGHT_NORMAL,
                '600': pango.WEIGHT_BOLD,
                '700': pango.WEIGHT_BOLD,
                '800': pango.WEIGHT_ULTRABOLD,
                '900': pango.WEIGHT_HEAVY,
                'normal': pango.WEIGHT_NORMAL,
                'bold': pango.WEIGHT_BOLD,
                } [sValue]
        except KeyError:
            warnings.warn("unknown font-style %s" % sValue)
        else:
            oTag.set_property("weight", iWeight)

    def _parse_style_font_family(self, oTag, sValue):
        """Change the font family."""
        oTag.set_property("family", sValue)

    def _parse_style_text_align(self, oTag, sValue):
        """Set the text alignment style."""
        try:
            iAlign = {
                'left': gtk.JUSTIFY_LEFT,
                'right': gtk.JUSTIFY_RIGHT,
                'center': gtk.JUSTIFY_CENTER,
                'justify': gtk.JUSTIFY_FILL,
                } [sValue]
        except KeyError:
            warnings.warn("Invalid text-align:%s requested" % sValue)
        else:
            oTag.set_property("justification", iAlign)

    def _parse_style_text_decoration(self, oTag, sValue):
        """Set the pango properties for the tag to match the desired html text
           decoration."""
        if sValue == "none":
            oTag.set_property("underline", pango.UNDERLINE_NONE)
            oTag.set_property("strikethrough", False)
        elif sValue == "underline":
            oTag.set_property("underline", pango.UNDERLINE_SINGLE)
            oTag.set_property("strikethrough", False)
        elif sValue == "overline":
            warnings.warn("text-decoration:overline not implemented")
            oTag.set_property("underline", pango.UNDERLINE_NONE)
            oTag.set_property("strikethrough", False)
        elif sValue == "line-through":
            oTag.set_property("underline", pango.UNDERLINE_NONE)
            oTag.set_property("strikethrough", True)
        elif sValue == "blink":
            warnings.warn("text-decoration:blink not implemented")
        else:
            warnings.warn("text-decoration:%s not implemented" % sValue)

    ## build a dictionary mapping styles to methods, for greater speed
    __style_methods = dict()
    for sStyle in ("background-color", "color", "font-family", "font-size",
                  "font-style", "font-weight", "margin-left", "margin-right",
                  "text-align", "text-decoration"):
        try:
            fMethod = locals()["_parse_style_%s" % sStyle.replace('-', '_')]
        except KeyError:
            warnings.warn("Style attribute '%s' not yet implemented" % sStyle)
        else:
            __style_methods[sStyle] = fMethod

    def _get_style_tags(self):
        """Get the current set of style tags"""
        return [oTag for oTag in self._aStyles if oTag is not None]

    def _begin_span(self, sStyle, oTag=None):
        """Start a <span> section"""
        if oTag is None:
            oTag = self._oTextBuf.create_tag()
        oTag.set_property("wrap-mode", gtk.WRAP_WORD)
        if sStyle is None:
            self._aStyles.append(oTag)
            return None
        for sAttr, sVal in [item.split(':', 1) for item in sStyle.split(';')]:
            sAttr = sAttr.strip().lower()
            sVal = sVal.strip()
            try:
                fMethod = self.__style_methods[sAttr]
            except KeyError:
                warnings.warn("Style attribute '%s' requested "
                              "but not yet implemented" % sAttr)
            else:
                fMethod(self, oTag, sVal)
        self._aStyles.append(oTag)

    def _end_span(self):
        """End a <span> section"""
        self._aStyles.pop(-1)

    def _insert_text(self, sText):
        """Insert text into the TextBuffer"""
        aTags = self._get_style_tags()
        if aTags:
            # pylint: disable-msg=W0142
            # * magic required
            self._oTextBuf.insert_with_tags(self._oIter, sText, *aTags)
        else:
            self._oTextBuf.insert(self._oIter, sText)

    def _flush_text(self):
        """Flush any pending text."""
        if not self._sText:
            return
        if self._oIter.starts_line():
            self._sText = self._sText.lstrip()
        self._insert_text(self._sText.replace('\n', ''))
        self._sText = ''

    # pylint: disable-msg=R0913
    # Arguments needed by function signature
    def _anchor_event(self, _oTag, _oView, oEvent, _oIter, oHref, oType):
        """Something happened to a link, so see if we need to react."""
        if oEvent.type == gtk.gdk.BUTTON_PRESS and oEvent.button == 1:
            self._oTextView.emit("url-clicked", oHref, oType)
            return True
        return False

    # Arguments needed so this can be called via 'size-allocate' event
    def __parse_length_frac_cb(self, oTextView, oAllocation,
            fFrac, fCallback, args):
        """call the required callback function when the size allocation
           changes."""
        # pylint: disable-msg=W0142
        # *magic required here
        fCallback(oAllocation.width * fFrac, *args)

    # pylint: enable-msg=R0913

    def handle_data(self, sContent):
        """Process the character comments of an element."""
        if self._bInTitle:
            return
        self._sText += WHITE_SPACE_REGEX.sub(' ', sContent)

    def get_targets(self):
        """Get the list of target anchors."""
        return self._dTargets

    # pylint: disable-msg=R0912, R0915, R0914
    # entity handler, so it is is a massive if..elif.. statement
    def handle_entityref(self, sEntity):
        """Convert entity to something we can use"""
        # This looks like all that textile will produce as entities
        if sEntity == 'lt':
            self.handle_data('<')
        elif sEntity == 'gt':
            self.handle_data('>')
        elif sEntity == 'amp':
            self.handle_data('&')
        elif sEntity == 'quot':
            self.handle_data('"')
        #else:
        #    print 'Unknown entity', sEntity

    # handle charrefs produced by textile as great if statement
    def handle_charref(self, sCharref):
        """Convert charref to something we can use"""
        self.handle_data(unichr(int(sCharref)))

    # start tag handler, so it is is a massive if..elif.. statement
    def handle_starttag(self, sName, oAttrs):
        """Parser start element handler"""
        oAttrs = dict(oAttrs)
        self._flush_text()
        try:
            oStyle = oAttrs['style']
        except KeyError:
            oStyle = None

        oTag = None
        if sName == 'a':
            oTag = self._oTextBuf.create_tag()
            oTag.set_property('foreground', '#0000ff')
            oTag.set_property('underline', pango.UNDERLINE_SINGLE)
            try:
                oType_ = oAttrs['type']
            except KeyError:
                oType_ = None
            if oAttrs.has_key('href'):
                oTag.connect('event', self._anchor_event, oAttrs['href'],
                        oType_)
                oTag.is_anchor = True
            if oAttrs.has_key('name'):
                # Add it to the list of valid targets
                oMark = self._oTextBuf.create_mark(None, self._oIter, True)
                self._dTargets[oAttrs['name']] = oMark
        elif sName in HTML_HEADING_TAGS:
            oTag = self._oTextBuf.create_tag()
            oTag.set_property('underline', pango.UNDERLINE_SINGLE)
            if sName == 'h1':
                oTag.set_property('weight', pango.WEIGHT_HEAVY)
                oTag.set_property('scale', pango.SCALE_X_LARGE)
            elif sName == 'h2':
                oTag.set_property('weight', pango.WEIGHT_ULTRABOLD)
                oTag.set_property('scale', pango.SCALE_LARGE)
            elif sName == 'h3':
                oTag.set_property('weight', pango.WEIGHT_BOLD)
        elif sName == 'title':
            self._bInTitle = True
            return
        elif sName == 'em' or sName == 'i':
            oTag = self._oTextBuf.create_tag()
            oTag.set_property('style', pango.STYLE_ITALIC)
        elif sName == 'strong' or sName == 'b':
            oTag = self._oTextBuf.create_tag()
            oTag.set_property('weight', pango.WEIGHT_BOLD)
        elif sName == 'font':
            oTag = self._oTextBuf.create_tag()
            dFontSize = {
                '-2': pango.SCALE_X_SMALL,
                '-1': pango.SCALE_SMALL,
                '1': pango.SCALE_LARGE,
                '2': pango.SCALE_X_LARGE,
            }
            if 'size' in oAttrs and oAttrs['size'] in dFontSize:
                oTag.set_property('scale', dFontSize[oAttrs['size']])
        elif sName == 'li':
            # indent 2em per list
            oTag = self._oTextBuf.create_tag()
            oTextAttrs = self._get_current_attributes()
            fFontSize = oTextAttrs.font.get_size() / pango.SCALE
            iDepth = len(self._aListCounters)
            oTag.set_property('left-margin', 2.0 * iDepth * SCREEN_RESOLUTION
                    * fFontSize)

        self._begin_span(oStyle, oTag)

        if oAttrs.has_key('id'):
            # Add it to the list of valid targets
            oMark = self._oTextBuf.create_mark(None, self._oIter, True)
            self._dTargets[oAttrs['id']] = oMark

        if sName == 'br':
            pass  # handled in endElement
        elif sName == 'p':
            if not self._oIter.starts_line():
                self._insert_text("\n")
        elif sName == 'div':
            if not self._oIter.starts_line():
                self._insert_text("\n")
        elif sName == 'span':
            pass
        elif sName == 'ul':
            if not self._oIter.starts_line():
                self._insert_text("\n")
            self._aListCounters.append(None)
        elif sName == 'ol':
            if not self._oIter.starts_line():
                self._insert_text("\n")
            self._aListCounters.append(0)
        elif sName == 'li':
            if self._aListCounters[-1] is None:
                sListHead = unichr(0x2022)
            else:
                self._aListCounters[-1] += 1
                sListHead = "%i." % self._aListCounters[-1]
            self._sText = sListHead + ' '
        elif sName == 'img':
            # pylint: disable-msg=W0703
            # we want to catch all errors here
            try:
                oFile = self._fLinkLoader(oAttrs['src'])
                oLoader = gtk.gdk.PixbufLoader()
                oLoader.write(oFile.read())
                oLoader.close()
                oPixbuf = oLoader.get_pixbuf()
            except Exception:
                oPixbuf = None
                try:
                    sAlt = oAttrs['alt']
                except KeyError:
                    sAlt = "Broken image"
            if oPixbuf is not None:
                aTags = self._get_style_tags()
                if aTags:
                    oTmpMark = self._oTextBuf.create_mark(None, self._oIter,
                            True)

                self._oTextBuf.insert_pixbuf(self._oIter, oPixbuf)

                if aTags:
                    oStart = self._oTextBuf.get_iter_at_mark(oTmpMark)
                    for oTag in aTags:
                        self._oTextBuf.apply_tag(oTag, oStart, self._oIter)
                    self._oTextBuf.delete_mark(oTmpMark)
            else:
                self._insert_text("[IMG: %s]" % sAlt)
        elif sName == 'body':
            pass
        elif sName == 'a':
            pass
        elif sName == 'head':
            pass
        elif sName == 'html':
            pass
        elif sName in HTML_HEADING_TAGS:
            pass
        elif sName == 'em' or sName == 'i':
            pass
        elif sName == 'strong' or sName == 'b':
            pass
        elif sName == 'font':
            pass
        else:
            warnings.warn("Unhandled start tag '%s'" % sName)

    # end tag handler, so it is is a massive if..elif.. statement
    def handle_endtag(self, sName):
        """Handle the end of a tag"""
        if sName == 'title':
            self._bInTitle = False
            return
        self._flush_text()

        if sName == 'p':
            if not self._oIter.starts_line():
                self._insert_text("\n")
        elif sName == 'div':
            if not self._oIter.starts_line():
                self._insert_text("\n")
        elif sName == 'span':
            pass
        elif sName == 'body':
            pass
        elif sName == 'br':
            self._insert_text("\n")
        elif sName == 'ul':
            self._aListCounters.pop()
        elif sName == 'ol':
            self._aListCounters.pop()
        elif sName == 'li':
            if not self._oIter.starts_line():
                self._insert_text("\n")
        elif sName == 'img':
            pass
        elif sName == 'body':
            pass
        elif sName == 'html':
            pass
        elif sName == 'head':
            pass
        elif sName == 'a':
            pass
        elif sName in HTML_HEADING_TAGS:
            if not self._oIter.starts_line():
                self._insert_text("\n")
        elif sName == 'em' or sName == 'i':
            pass
        elif sName == 'strong' or sName == 'b':
            pass
        elif sName == 'font':
            pass
        else:
            warnings.warn("Unhandled end tag '%s'" % sName)

        self._end_span()


class HTMLTextView(gtk.TextView):
    # pylint: disable-msg=R0904
    # gtk.Widget, so many public methods
    """TextView subclass to display HTML"""
    __gtype_name__ = 'HTMLTextView'
    __gsignals__ = {
        'url-clicked': (gobject.SIGNAL_RUN_LAST, None, (str, str)),
        # href, type
    }

    def __init__(self, fLinkLoader):
        """TextView subclass to display HTML.

           fLinkLoader: function to load links with
                        (takes local URL, returns file-like object)
           """
        gtk.TextView.__init__(self)
        self.set_wrap_mode(gtk.WRAP_CHAR)
        self.set_editable(False)
        self._bChangedCursor = False
        self.connect("motion-notify-event", self.__motion_notify_event)
        self.connect("leave-notify-event", self.__leave_event)
        self.connect("enter-notify-event", self.__motion_notify_event)
        self.set_pixels_above_lines(3)
        self.set_pixels_below_lines(3)
        self._dTargets = {}
        self._fLinkLoader = fLinkLoader

    def __leave_event(self, oWidget, _oEvent):
        """Cursor has left the widget."""
        if self._bChangedCursor:
            oWindow = oWidget.get_window(gtk.TEXT_WINDOW_TEXT)
            oWindow.set_cursor(gtk.gdk.Cursor(gtk.gdk.XTERM))
            self._bChangedCursor = False

    def __motion_notify_event(self, oWidget, _oEvent):
        """Change the cursor if the pointer is over a link"""
        iXPos, iYPos, _oIgnore = oWidget.window.get_pointer()
        iXPos, iYPos = oWidget.window_to_buffer_coords(gtk.TEXT_WINDOW_TEXT,
                iXPos, iYPos)
        aTags = oWidget.get_iter_at_location(iXPos, iYPos).get_tags()
        for oTag in aTags:
            if getattr(oTag, 'is_anchor', False):
                bOverAnchor = True
                break
        else:
            bOverAnchor = False
        if not self._bChangedCursor and bOverAnchor:
            oWindow = oWidget.get_window(gtk.TEXT_WINDOW_TEXT)
            oWindow.set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND2))
            self._bChangedCursor = True
        elif self._bChangedCursor and not bOverAnchor:
            oWindow = oWidget.get_window(gtk.TEXT_WINDOW_TEXT)
            oWindow.set_cursor(gtk.gdk.Cursor(gtk.gdk.XTERM))
            self._bChangedCursor = False
        return False

    def set_text_pos(self, sTextAnchor):
        """Set the position to the anchor sTextAnchor"""
        if self._dTargets.has_key(sTextAnchor):
            oMark = self._dTargets[sTextAnchor]
            self.scroll_to_mark(oMark, 0, True)
        # We just silently ignore invalid anchors

    def display_html(self, fHTMLInput):
        """Display the HTML from the file-like object fHTMLInput"""
        oBuffer = self.get_buffer()
        oStartOfBuf, oEndOfBuf = oBuffer.get_bounds()
        oBuffer.delete(oStartOfBuf, oEndOfBuf)
        oEndOfBuf = oBuffer.get_end_iter()

        oHandler = HtmlHandler(self, oEndOfBuf, self._fLinkLoader)
        oHandler.feed(fHTMLInput.read())

        if not oEndOfBuf.starts_line():
            oBuffer.insert(oEndOfBuf, "\n")

        # Need the anchor -> tag mappings
        self._dTargets = oHandler.get_targets()


class HTMLViewDialog(SutekhDialog):
    # pylint: disable-msg=R0904, R0902
    # R0904: gtk.Widget, so many public methods
    # R0902: We need to keep a lot of state to handle navigation
    """Dialog Window that wraps the HTMLTextView

       Used to show HTML Manuals in Sutekh.
       """

    _sError = """<html>
    <body>
    <h1>Resource not found</h1>
    <p>Unable to load the resource %(missing)s</p>
    </body>
    </html>
    """

    def __init__(self, oParent, fInput, fLinkLoader):
        """Dialog Window that wraps the HTMLTextView

           fInput: file-like object to read the HTML from.
           fLinkLoader: function to load links with (passed to HTMLTextView)
           """
        super(HTMLViewDialog, self).__init__('Help', oParent,
                oButtons=(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
        # pylint: disable-msg=E1101
        # vbox confuses pylint
        oDirButtons = gtk.HButtonBox()
        self._oBackButton = gtk.Button(stock=gtk.STOCK_GO_BACK)
        self._oBackButton.connect('pressed', self._go_back)
        self._oForwardButton = gtk.Button(stock=gtk.STOCK_GO_FORWARD)
        self._oForwardButton.connect('pressed', self._go_forward)
        oDirButtons.pack_start(self._oBackButton)
        oDirButtons.pack_start(self._oForwardButton)
        self._oBackButton.set_sensitive(False)
        self._oForwardButton.set_sensitive(False)
        self._fCurrent = fInput
        self._fLinkLoader = fLinkLoader
        self._sTextAnchor = None
        self._aPastUrls = []
        self._aFutureUrls = []
        self.vbox.pack_start(oDirButtons, False, False)
        self._oHTMLTextView = HTMLTextView(self._fLinkLoader)
        self._oView = AutoScrolledWindow(self._oHTMLTextView)
        self.set_default_size(400, 600)
        self.vbox.pack_start(self._oView, True,
                True)
        self._oHTMLTextView.connect('url-clicked', self._url_clicked)
        self.connect('response', lambda x, but: self.hide())
        # ignore the delete event (response handler will hide)
        self.connect('delete-event', lambda x, dlg: True)
        self._update_view()

    def _update_view(self):
        """Redraw the pane with the contents of self._fCurrent"""
        self._fCurrent.seek(0)
        self._oHTMLTextView.display_html(self._fCurrent)
        if self._sTextAnchor:
            self._oHTMLTextView.set_text_pos(self._sTextAnchor)
        if len(self._aPastUrls) > 0:
            self._oBackButton.set_sensitive(True)
        else:
            self._oBackButton.set_sensitive(False)
        if len(self._aFutureUrls) > 0:
            self._oForwardButton.set_sensitive(True)
        else:
            self._oForwardButton.set_sensitive(False)
        self.show_all()

    def show_page(self, fInput, sPos=None):
        """Display the html file in fInput in the current window."""
        self._aPastUrls.append((self._fCurrent, self._sTextAnchor))
        self._aFutureUrls = []  # Forward history is lost
        if fInput:
            self._fCurrent = fInput
        self._sTextAnchor = sPos
        self._update_view()

    def _url_clicked(self, _oWidget, sUrl, _oType):
        """Update the HTML widget with the new url"""
        aParts = sUrl.split('#')
        sFile = aParts[0]
        if len(aParts) > 1:
            sPos = aParts[1]  # Get anchor
        else:
            sPos = None
        if sFile:
            # pylint: disable-msg=W0703
            # we really do want all the exceptions
            try:
                fInput = self._fLinkLoader(sFile)
            except Exception, _oExp:
                sError = self._sError % {'missing': sUrl}
                fInput = StringIO(sError)
                sPos = None
            self.show_page(fInput, sPos)
        else:
            self.show_page(None, sPos)

    def _go_back(self, _oWidget):
        """Go backwards through the list of visited urls"""
        if len(self._aPastUrls) == 0:
            return
        self._aFutureUrls.append((self._fCurrent, self._sTextAnchor))
        self._fCurrent, self._sTextAnchor = self._aPastUrls.pop()
        self._update_view()

    def _go_forward(self, _oWidget):
        """Go forward through the list of visited urls"""
        if len(self._aFutureUrls) == 0:
            return
        self._aPastUrls.append((self._fCurrent, self._sTextAnchor))
        self._fCurrent, self._sTextAnchor = self._aFutureUrls.pop()
        self._update_view()



# test_DatabaseVersion.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2009 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test database versioning code"""

from sutekh.tests.TestCore import SutekhTest
from sutekh.core.DatabaseVersion import DatabaseVersion
from sutekh.core.SutekhObjects import TABLE_LIST


class DatabaseVersionTests(SutekhTest):
    """Class for the database version tests."""

    def test_version(self):
        """Handle the expected case where everything is fine"""
        oVersion = DatabaseVersion()

        aVersions = [oTable.tableversion for oTable in TABLE_LIST]
        # Test that this works as expected
        self.assertTrue(oVersion.check_tables_and_versions(TABLE_LIST,
            aVersions))
        # Test for failures
        oTable = TABLE_LIST[0]
        self.assertFalse(oVersion.check_tables_and_versions([oTable],
            [oTable.tableversion - 1]))
        self.assertFalse(oVersion.check_tables_and_versions([oTable],
            [oTable.tableversion + 1]))

        for oTable in TABLE_LIST[0:6]:
            # Test some tables
            aVersions = range(max(0, oTable.tableversion - 1),
                    oTable.tableversion + 2)
            aVersionsFail = range(oTable.tableversion + 1,
                    oTable.tableversion + 5)
            # Check with cache expired
            DatabaseVersion.expire_cache()
            self.assertTrue(oVersion.check_table_in_versions(oTable,
                aVersions))
            self.assertFalse(oVersion.check_table_in_versions(oTable,
                aVersionsFail))
            # Test with the cache filled
            self.assertTrue(oVersion.check_table_in_versions(oTable,
                aVersions))
            self.assertFalse(oVersion.check_table_in_versions(oTable,
                aVersionsFail))

        aTables = [TABLE_LIST[0], TABLE_LIST[1], TABLE_LIST[5]]
        aVersions = [TABLE_LIST[0].tableversion + 1,
                TABLE_LIST[1].tableversion + 3,
                TABLE_LIST[5].tableversion - 1]

        aLowerTables, aHigherTables = \
                oVersion.get_bad_tables(aTables, aVersions)

        # aLowerTables -> Database version < requested, so these are the
        # expected results
        self.assertEqual(aLowerTables, [TABLE_LIST[0].sqlmeta.table,
            TABLE_LIST[1].sqlmeta.table])
        self.assertEqual(aHigherTables, [TABLE_LIST[5].sqlmeta.table])



# WriteVEKNForum.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2011 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

# pylint: disable-msg=C0301
# documentation, so line length ignored
"""Writer for VEKN Forum posts, based on the ARDB Text format

   Style intended to copy that produced by ARDB, based on the TWD postings
   on the vekn.net forum.

   Example deck:

   [size=18][b]Deck Name : Followers of Set Preconstructed Deck[/b][/size]
   [b][u]Author :[/u][/b] L. Scott Johnson
   [b][u]Description :[/u][/b]
   Followers of Set Preconstructed Starter from Lords of the Night.

   http://www.white-wolf.com/vtes/index.php?line=Checklist_LordsOfTheNight

   [size=18][u]Crypt [12 vampires] Capacity min: 2 max: 10 average: 5.84[/u][/size]
   [table]
   [tr][td]2x[/td][td][url=http://www.secretlibrary.info/...]Nakhthorheb[/url][/td][td][/td][td](10)[/td][td]:OBF: :PRE: :SER:[/td][td]:fose: Follower of Set[/td][td](group 4)[/td][/tr]
   ...
   [/table]

   [size=18][u]Library [77 cards][/u][/size]
   [b][u]Action [20][/u][/b]
     2x [url=...]Blithe Acceptance[/url]
     4x [url=....]Dream World[/url]
   ...
   """
# pylint: enable-msg=C0301
import time
from sutekh.core.ArdbInfo import ArdbInfo
from sutekh.SutekhUtility import secret_library_url
from sutekh.SutekhInfo import SutekhInfo


def add_clan_symbol(dLine):
    """Fix the clan symbol to account for special cases"""
    dSpecialClanMap = {
            'True Brujah': ':trub:',
            'Daughter of Cacophony': ':doca:',
            'Follower of Set': ':fose:',
            'Harbinger of Skulls': ':hosk:',
            'Blood Brother': ':bbro:',
            }
    if dLine['clan'] in dSpecialClanMap:
        sSymbol = dSpecialClanMap[dLine['clan']]
    elif 'antitribu' in dLine['clan']:
        sSymbol = "!%s!" % dLine['clan'][:4].lower()
    else:
        # This is the usual default
        sSymbol = ":%s:" % dLine['clan'][:4].lower()
    dLine['clansymbol'] = sSymbol


class WriteVEKNForum(ArdbInfo):
    """Create a string suitable for pasting into the VEKN forums"""

    # pylint: disable-msg=R0201
    # method for consistency with the other methods
    def _gen_header(self, oHolder):
        """Generate an suitable forum header."""
        return "[size=18][b]Deck Name : %s[/b][/size]\n" \
               "[b][u]Author :[/u][/b] %s\n" \
               "[b][u]Description :[/u][/b]\n%s\n" % (oHolder.name,
                       oHolder.author, oHolder.comment)
    # pylint: enable-msg=R0201

    def _gen_crypt(self, dCards):
        """Generaten a VEKN Forum crypt description.

           dCards is mapping of (card id, card name) -> card count.
           """
        (dVamps, dCryptStats) = self._extract_crypt(dCards)
        dCombinedVamps = self._group_sets(dVamps)

        sCrypt = "[size=18][u]Crypt [%(size)d vampires] Capacity " \
                "min: %(min)d max: %(max)d average: %(avg).2f[/u][/size]\n" \
                % dCryptStats

        aCryptLines = []
        # ARDB's discipline & title padding are based on the longest entry
        # so we need to keep track and format later
        for oCard,  (iCount, _sSet) in sorted(dCombinedVamps.iteritems(),
                key=lambda x: (-x[1][0], -max(x[0].capacity, x[0].life),
                    x[0].name)):
            # We sort inversely on count, then capacity and then normally by
            # name
            dLine = {'count': iCount}
            if len(oCard.creed) > 0:
                dLine['clan'] = "%s (Imbued)" % \
                        [x.name for x in oCard.creed][0]
                dLine['capacity'] = oCard.life
            else:
                dLine['clan'] = [x.name for x in oCard.clan][0]
                dLine['capacity'] = oCard.capacity
            add_clan_symbol(dLine)
            dLine['name'] = oCard.name
            dLine['url'] = secret_library_url(oCard, True)
            dLine['adv'] = ''
            if oCard.level is not None:
                dLine['name'] = dLine['name'].replace(' (Advanced)', '')
                dLine['adv'] = 'Adv'
            aDisc = self._gen_disciplines(oCard).split()
            if aDisc:
                # Get nice frum symbols for the disciplines
                dLine['disc'] = ":" + ": :".join(aDisc) + ":"
                # Annoying special cases
                dLine['disc'] = dLine['disc'].replace(":jud:", ":jus:")
                dLine['disc'] = dLine['disc'].replace(":vis:", ":vsn:")
                dLine['disc'] = dLine['disc'].replace(":FLI:", ":flight:")
            else:
                dLine['disc'] = ""

            dLine['title'] = '   '
            if len(oCard.title):
                dLine['title'] = [oC.name for oC in oCard.title][0]
                dLine['title'] = dLine['title'].replace('Independent with ',
                        '')[:12]
            dLine['group'] = int(oCard.group)
            aCryptLines.append(dLine)

        sCrypt += "[table]\n"
        for dLine in aCryptLines:
            sCrypt += "[tr][td]%(count)dx[/td]" \
                    "[td][url=%(url)s]%(name)s[/url][/td][td]%(adv)s[/td]" \
                    "[td](%(capacity)d)[/td][td]%(disc)s[/td]" \
                    "[td]%(title)s[/td][td]%(clansymbol)s %(clan)s[/td]" \
                    "[td](group %(group)d)[/td][/tr]\n" % dLine

        sCrypt += "[/table]"

        return sCrypt

    def _gen_library(self, dCards):
        """Generaten an VEKN Forum library description.

           dCards is mapping of (card id, card name) -> card count.
           """
        (dLib, iLibSize) = self._extract_library(dCards)
        dCombinedLib = self._group_sets(dLib)

        sLib = "[size=18][u]Library [%d cards][/u][/size]\n" \
            % (iLibSize,)

        dTypes = {}
        for oCard, (iCount, sTypeString, _sSet) in dCombinedLib.iteritems():
            if sTypeString not in dTypes:
                dTypes[sTypeString] = {}
            dTypes[sTypeString].setdefault(oCard, 0)
            dTypes[sTypeString][oCard] += iCount

        for sTypeString in sorted(dTypes):
            dCards = dTypes[sTypeString]
            iTotal = sum(dCards.values())

            sLib += "[b][u]%s [%d][/u][/b]\n" % (sTypeString, iTotal)

            for oCard, iCount in sorted(dCards.iteritems(),
                    key=lambda x: x[0].name):
                sUrl = secret_library_url(oCard, False)
                sLib += " %dx [url=%s]%s[/url]\n" % (iCount, sUrl, oCard.name)

            sLib += "\n"

        return sLib

    # pylint: disable-msg=R0913
    # we need all these arguments
    def write(self, fOut, oHolder):
        """Takes filename, deck details and a dictionary of cards, of the
           form dCard[(id, name, set)] = count and writes the file."""
        # We don't encode strange characters, and just write the unicode string
        # to file. This looks to match ARDB's behaviour (tested against
        # ARDB version 2.8
        dCards = self._get_cards(oHolder.cards)
        fOut.write(self._gen_header(oHolder))
        fOut.write("\n")
        fOut.write(self._gen_crypt(dCards))
        fOut.write("\n")
        fOut.write(self._gen_library(dCards))
        fOut.write("\n")
        fOut.write("Recorded with : Sutekh %s [ %s ]\n" %
                (SutekhInfo.VERSION_STR,
                    time.strftime('%Y-%m-%d', time.localtime())))

    # pylint: enable-msg=R0913



# test_GuessFileParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2009 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test guessing the file format"""

import unittest
from sutekh.tests.TestCore import SutekhTest
from sutekh.io.GuessFileParser import GuessFileParser
from sutekh.io.AbstractCardSetParser import AbstractCardSetParser
from sutekh.io.PhysicalCardSetParser import PhysicalCardSetParser
from sutekh.io.ARDBXMLDeckParser import ARDBXMLDeckParser
from sutekh.io.ARDBXMLInvParser import ARDBXMLInvParser
from sutekh.io.ARDBTextParser import ARDBTextParser
from sutekh.io.JOLDeckParser import JOLDeckParser
from sutekh.io.ELDBInventoryParser import ELDBInventoryParser
from sutekh.io.ELDBDeckFileParser import ELDBDeckFileParser
from sutekh.io.ELDBHTMLParser import ELDBHTMLParser
from sutekh.io.LackeyDeckParser import LackeyDeckParser
from sutekh.tests.io.test_JOLDeckParser import JOL_EXAMPLE_1
from sutekh.tests.io.test_ARDBTextParser import ARDB_TEXT_EXAMPLE_1
from sutekh.tests.io.test_ARDBXMLDeckParser import ARDB_DECK_EXAMPLE_1
from sutekh.tests.io.test_ARDBXMLInvParser import ARDB_INV_EXAMPLE_1
from sutekh.tests.io.test_ELDBHTMLParser import ELDB_HTML_EXAMPLE_1
from sutekh.tests.io.test_ELDBInventoryParser import ELDB_INV_EXAMPLE_1
from sutekh.tests.io.test_ELDBDeckFileParser import ELDB_TEXT_EXAMPLE_1
from sutekh.tests.io.test_LackeyDeckParser import LACKEY_EXAMPLE_1
from sutekh.tests.io.test_AbstractCardSetParser import ACS_EXAMPLE_1
from sutekh.tests.io.test_PhysicalCardSetParser import PCS_EXAMPLE_1


class TestGuessFileParser(SutekhTest):
    """class for the guess file parser tests"""

    TESTS = [
            (AbstractCardSetParser, ACS_EXAMPLE_1),
            (PhysicalCardSetParser, PCS_EXAMPLE_1),
            (ARDBXMLDeckParser, ARDB_DECK_EXAMPLE_1),
            (ARDBXMLInvParser, ARDB_INV_EXAMPLE_1),
            (ARDBTextParser, ARDB_TEXT_EXAMPLE_1),
            (JOLDeckParser, JOL_EXAMPLE_1),
            (ELDBInventoryParser, ELDB_INV_EXAMPLE_1),
            (ELDBDeckFileParser, ELDB_TEXT_EXAMPLE_1),
            (ELDBHTMLParser, ELDB_HTML_EXAMPLE_1),
            (LackeyDeckParser, LACKEY_EXAMPLE_1),
            ]

    def test_guess(self):
        """test guess against the correct parser."""
        oGuessParser = GuessFileParser()
        for cCorrectParser, sData in self.TESTS:
            oHolder1 = self._make_holder_from_string(oGuessParser, sData)
            oHolder2 = self._make_holder_from_string(cCorrectParser(),
                    sData)

            self.assertEqual(oHolder1.name, oHolder2.name,
                    "Holder names don't match:"
                    " %s vs %s, using %s, correct choice %s" % (oHolder1.name,
                        oHolder2.name, oGuessParser.oChosenParser,
                        cCorrectParser))
            self.assertEqual(oHolder1.get_cards(), oHolder2.get_cards(),
                    "Guess cards don't match:"
                    " %s vs %s, using %s, correct %s" % (oHolder1.get_cards(),
                        oHolder2.get_cards(), oGuessParser.oChosenParser,
                        cCorrectParser))

if __name__ == "__main__":
    unittest.main()



# WriteArdbInvXML.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2006 Neil Muller <drnlmuller+sutekh@gmail.com>
# Based off the Anarach Revolt Deck Builder xml support,
# ARDB (c) Francios Gombalt, Christoph Boget, Ville Virta and Vincent Ripoll
# GPL - see COPYING for details

"""Given a list of Abstract Cards in a set, write a XML file compatable with
   the Anarch Revolt Deck Builder's XML inventory format."""

from sutekh.io.WriteArdbXML import WriteArdbXML
# pylint: disable-msg=E0611, F0401
# xml.etree is a python2.5 thing
try:
    from xml.etree.ElementTree import Element, SubElement
except ImportError:
    from elementtree.ElementTree import Element, SubElement
# pylint: enable-msg=E0611, F0401


class WriteArdbInvXML(WriteArdbXML):
    """Reformat cardset to elementTree and export it to a ARDB
       compatible XML Inventory file."""

    def _gen_tree(self, oHolder):
        """Creates the actual XML document into memory."""
        dCards = self._get_cards(oHolder.cards)
        dVamps, dCryptStats = self._extract_crypt(dCards)
        dLib, iLibSize = self._extract_library(dCards)

        oRoot = Element('inventory')
        self._add_date_version(oRoot)

        oCryptElem = SubElement(oRoot, 'crypt', size=str(dCryptStats['size']))
        dCombinedVamps = self._group_sets(dVamps)
        self.format_vamps(oCryptElem, dCombinedVamps)
        oLibElem = SubElement(oRoot, 'library', size=str(iLibSize))
        dCombinedLib = self._group_sets(dLib)
        self.format_library(oLibElem, dCombinedLib)
        return oRoot

    def format_vamps(self, oCryptElem, dCombinedVamps):
        """Convert the Vampire dictionary into ElementTree representation."""
        for oCard, (iNum, sSet) in sorted(dCombinedVamps.iteritems(),
                key=lambda x: (x[0].name, x[1][1], x[1][0])):
            # This won't match the ARDB ID's, unless by chance.
            # It looks like that should not be an issue as ARDB will
            # use the name if the IDs don't match
            oCardElem = SubElement(oCryptElem, 'vampire',
                    databaseID=str(oCard.id), have=str(iNum),
                    spare='0', need='0')
            self._ardb_crypt_card(oCardElem, oCard, sSet)

    def format_library(self, oLibElem, dCombinedLib):
        """Format the dictionary of library cards for the element tree."""
        for oCard, (iNum, _sType, sSet) in sorted(dCombinedLib.iteritems(),
                key=lambda x: (x[0].name, x[1][2], x[1][0])):
            oCardElem = SubElement(oLibElem, 'card', databaseID=str(oCard.id),
                    have=str(iNum), spare='0', need='0')
            self._ardb_lib_card(oCardElem, oCard, sSet)



# test_WriteArdbInvXML.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test Writing a card set to an ARDB inventory XML file"""

from sutekh.tests.TestCore import SutekhTest
from sutekh.tests.core.test_PhysicalCardSet import make_set_1
from sutekh.core.CardSetHolder import CardSetWrapper
from sutekh.io.WriteArdbInvXML import WriteArdbInvXML
from sutekh.SutekhInfo import SutekhInfo
import unittest
import time

# pylint: disable-msg=W0511, C0301
# W0511 - this is not a actual TODO item
# C0301 - Ignore line length limits for this string
EXPECTED_1 = """<inventory databaseVersion="%s" formatVersion="-TODO-1.0" generator="Sutekh [ %s ]">
  <date>DATE</date>
  <crypt size="4">
    <vampire databaseID="11" have="1" need="0" spare="0">
      <adv />
      <name>Abebe</name>
      <set>LoB</set>
    </vampire><vampire databaseID="20" have="1" need="0" spare="0">
      <adv>Advanced</adv>
      <name>Alan Sovereign</name>
      <set>Promo20051001</set>
    </vampire><vampire databaseID="42" have="1" need="0" spare="0">
      <adv />
      <name>Inez "Nurse216" Villagrande</name>
      <set>NoR</set>
    </vampire><vampire databaseID="63" have="1" need="0" spare="0">
      <adv />
      <name>Siamese, The</name>
      <set>BL</set>
    </vampire>
  </crypt><library size="13">
    <card databaseID="1" have="4" need="0" spare="0">
      <name>.44 Magnum</name>
      <set>CE</set>
    </card><card databaseID="2" have="2" need="0" spare="0">
      <name>AK-47</name>
      <set>LotN</set>
    </card><card databaseID="6" have="1" need="0" spare="0">
      <name>Aaron's Feeding Razor</name>
      <set>KoT</set>
    </card><card databaseID="8" have="2" need="0" spare="0">
      <name>Abbot</name>
      <set>Third</set>
    </card><card databaseID="14" have="2" need="0" spare="0">
      <name>Abombwe</name>
      <set>LoB</set>
    </card><card databaseID="60" have="1" need="0" spare="0">
      <name>Scapelli, The Family "Mechanic"</name>
      <set>DS</set>
    </card><card databaseID="52" have="1" need="0" spare="0">
      <name>Path of Blood, The</name>
      <set>LotN</set>
    </card>
  </library>
</inventory>""" % (WriteArdbInvXML.sDatabaseVersion, SutekhInfo.VERSION_STR)
# pylint: enable-msg=W0511, C0301


class ArdbInvXMLWriterTests(SutekhTest):
    """class for the ARDB inventory XML writer tests"""

    def test_deck_writer(self):
        """Test ARDB XML inventory writing"""
        oPhysCardSet1 = make_set_1()
        sCurDate = time.strftime('>%Y-%m-%d<', time.localtime())

        # Check output

        oWriter = WriteArdbInvXML()
        sData = self._round_trip_obj(oWriter, CardSetWrapper(oPhysCardSet1))
        sData = sData.replace(sCurDate, '>DATE<')

        self.assertEqual(sData, EXPECTED_1)

if __name__ == "__main__":
    unittest.main()



# ExpansionStats.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2010 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Plugin for analysing all expansions and reporting the number of cards
   of each rarity."""

import gtk
import pango
import gobject
from sutekh.core.SutekhObjects import PhysicalCard, AbstractCard
from sutekh.core.Groupings import ExpansionRarityGrouping
from sutekh.core.Filters import make_illegal_filter, NullFilter
from sutekh.gui.PluginManager import SutekhPlugin
from sutekh.gui.SutekhDialog import SutekhDialog
from sutekh.gui.AutoScrolledWindow import AutoScrolledWindow


class ExpansionStats(SutekhPlugin):
    """Display card counts and stats for each expansion, rarity grouping.

       A dialog listing cards for each expansion, split by the rarity,
       with a special category for cards only included in the precon decks.
       The cards are grouped by the current grouping used by the WW card
       list view.
       """

    dTableVersions = {}
    aModelsSupported = (PhysicalCard,)

    # pylint: disable-msg=W0142
    # **magic OK here
    def __init__(self, *args, **kwargs):
        super(ExpansionStats, self).__init__(*args, **kwargs)
        self._oStatsVbox = None

    def get_menu_item(self):
        """Register on the 'Analyze' menu"""
        if not self.check_versions() or not self.check_model_type():
            return None
        oExpStats = gtk.MenuItem("Expansion Stats")
        oExpStats.connect("activate", self.activate)
        return ('Analyze', oExpStats)

    def activate(self, _oWidget):
        """Handle response from menu"""
        oDlg = self.make_dialog()
        oDlg.run()
        self._oStatsVbox = None

    def make_dialog(self):
        """Create the dialog to display the statistics"""
        oDlg = SutekhDialog("Expansion Statistics", self.parent,
                gtk.DIALOG_DESTROY_WITH_PARENT)

        oDlg.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE)
        oDlg.connect("response", lambda oW, oR: oDlg.destroy())

        self._oStatsVbox = gtk.VBox(False, 0)

        # pylint: disable-msg=E1101
        # vbox methods not seen
        oDlg.vbox.pack_start(self._oStatsVbox)
        oDlg.set_size_request(600, 400)
        oDlg.show_all()

        self.populate_stats_box()

        return oDlg

    def populate_stats_box(self):
        """Create a tree view of stats in self._oStatsVbox."""
        # clean box
        for oChild in self._oStatsVbox.get_children():
            self._oStatsVbox.remove(oChild)

        oView = StatsView(self.model.groupby, self.model.hideillegal)

        # top align, using viewport to scroll
        self._oStatsVbox.pack_start(AutoScrolledWindow(oView, True))
        self._oStatsVbox.show_all()


class StatsView(gtk.TreeView):
    # pylint: disable-msg=R0904
    # gtk classes, so we have lots of public methods
    """TreeView used to display expansion stats"""

    def __init__(self, cGrping, bHideIllegal):
        self._oModel = StatsModel(cGrping, bHideIllegal)
        self._aLabels = ["Expansion", "Count"]

        super(StatsView, self).__init__(self._oModel)

        oCell = gtk.CellRendererText()
        oCell.set_property('style', pango.STYLE_ITALIC)

        for iCol, sLabel in enumerate(self._aLabels):
            oColumn = gtk.TreeViewColumn(sLabel, oCell, text=iCol)
            oColumn.set_sort_column_id(iCol)
            self.append_column(oColumn)

        self.set_grid_lines(gtk.TREE_VIEW_GRID_LINES_BOTH)


class StatsModel(gtk.TreeStore):
    # pylint: disable-msg=R0904
    # gtk classes, so we have lots of public methods
    """TreeStore to hold the data about the expansion statistics"""

    def __init__(self, cGrping, bHideIllegal):
        # pylint: disable-msg=W0142
        # We need the * magic here
        super(StatsModel, self).__init__(gobject.TYPE_STRING,
                gobject.TYPE_INT)
        self.oLegalFilter = NullFilter()
        if bHideIllegal:
            self.oLegalFilter = make_illegal_filter()
        self.load(cGrping)

    # pylint: disable-msg=R0914
    # R0914: We use lots of local variables for clarity
    def load(self, cSubGrping):
        """Populate the contents of the TreeStore"""
        self.clear()

        aCards = self.oLegalFilter.select(AbstractCard)
        oGrouping = ExpansionRarityGrouping(aCards)
        aTopLevel = []
        oExpIter = None
        iTotal = 0
        for sGroup, oGroupIter in sorted(oGrouping):
            if sGroup != 'Promo':
                sExp, sRarity = sGroup.split(':')
            else:
                sExp, sRarity = 'Promo', None
            if sExp not in aTopLevel:
                oExpIter = self.append(None)
                self.set(oExpIter, 0, sExp.strip())
                aTopLevel.append(sExp)
                iTotal = 0
            aSubCards = list(oGroupIter)
            if sRarity:
                oIter = self.append(oExpIter)
                self.set(oIter, 0, sRarity.strip())
                if sRarity.strip() != 'Precon Only':
                    iTotal += len(aSubCards)
                    self.set(oExpIter, 1, iTotal)
            else:
                oIter = oExpIter
            self.set(oIter, 1, len(aSubCards))
            oSubGroup = cSubGrping(aSubCards)
            for sInfo, oSubGrpIter in sorted(oSubGroup):
                oChildIter = self.append(oIter)
                if not sInfo:
                    sInfo = "<< None >>"
                self.set(oChildIter, 0, sInfo, 1, len(list(oSubGrpIter)))
                for sCard in sorted([x.name for x in oSubGrpIter]):
                    oCardIter = self.append(oChildIter)
                    self.set(oCardIter, 0, sCard, 1, 1)

plugin = ExpansionStats



# test_ELDBHTMLParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test case for ELDB HTML parser"""

from sutekh.io.ELDBHTMLParser import ELDBHTMLParser
from sutekh.tests.TestCore import SutekhTest
import unittest

ELDB_HTML_EXAMPLE_1 = """<HTML>
<HEAD>
<TITLE>Test Deck</TITLE>
</HEAD>
<BODY BGCOLOR="#ffffff">
<TABLE WIDTH=650 ALIGN="center" CELLSPACING=0 CELLPADDING=0 BORDER=0>
<TR><TD WIDTH=130>Deck Name:</TD><TD WIDTH=520>Test Deck</TD></TR>
<TR><TD WIDTH=130>Created By:</TD><TD WIDTH=520>Anon Y Mous</TD></TR>
<TR><TD WIDTH=130 VALIGN="top">Description:</TD><TD WIDTH=520>Simple test deck.</TD></TR>
<TR><TD COLSPAN=2 WIDTH=650>&nbsp;</TD><TR>
<TR><TD COLSPAN=2 WIDTH=650 BGCOLOR="#eeeeee">Crypt: (3 cards, Min: 2, Max: 4, Avg: 4.67)</TD></TR>
<TR><TD COLSPAN=2 WIDTH=650>1&nbsp;&nbsp;<a href="http://monger.vekn.org/showvamp.html?ID=123" class="textLink">Test Vamp 1</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;aus FOR OBE&nbsp;&nbsp;6,&nbsp;&nbsp;Salubri</TD></TR>
<TR><TD COLSPAN=2 WIDTH=650>2&nbsp;&nbsp;<a href="http://monger.vekn.org/showvamp.html?ID=321" class="textLink">Test Vamp 2</a>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;ani FOR PRO&nbsp;&nbsp;5,&nbsp;&nbsp;Gangrel</TD></TR>
<TR><TD COLSPAN=2 WIDTH=650>&nbsp;</TD></TR>
<TR><TD COLSPAN=2 WIDTH=650 BGCOLOR="#eeeeee">Library: (7 cards)</TD></TR>
<TR><TD COLSPAN=2 WIDTH=650>Master (2 cards)</TD></TR>
<TR><TD COLSPAN=2 WIDTH=650>2&nbsp;&nbsp;<a href="http://monger.vekn.org/showcard.html?ID=111" class="textLink">Test Card 1</a></TD></TR>
<TR><TD COLSPAN=2 WIDTH=650>&nbsp;</TD></TR>
<TR><TD COLSPAN=2 WIDTH=650>Action (5 cards)</TD></TR>
<TR><TD COLSPAN=2 WIDTH=650>3&nbsp;&nbsp;<a href="http://monger.vekn.org/showcard.html?ID=222" class="textLink">Test Card 2</a></TD></TR>
<TR><TD COLSPAN=2 WIDTH=650>2&nbsp;&nbsp;<a href="http://monger.vekn.org/showcard.html?ID=3333" class="textLink">Test Card 3</a></TD></TR>
<TR><TD COLSPAN=2 WIDTH=650>&nbsp;</TD></TR>
<TR><TD COLSPAN=2 WIDTH=650 ALIGN="right">This file was last saved at 11:29:00 PM  on 01/09/08 </TD></TR>
</TABLE>
</BODY>
</HTML>
"""


class ELDBHTMLParserTests(SutekhTest):
    """class for the ELDB HTML input parser"""

    def test_basic(self):
        """Run the input test."""
        oHolder = self._make_holder_from_string(ELDBHTMLParser(),
                ELDB_HTML_EXAMPLE_1)

        self.assertEqual(oHolder.name, "Test Deck")
        self.assertEqual(oHolder.author, "Anon Y Mous")
        self.failUnless(oHolder.comment.startswith("Simple test deck."))

        aCards = oHolder.get_cards()

        self.assertEqual(len(aCards), 5)
        self.failUnless(("Test Vamp 1", 1) in aCards)
        self.failUnless(("Test Vamp 2", 2) in aCards)
        self.failUnless(("Test Card 1", 2) in aCards)
        self.failUnless(("Test Card 2", 3) in aCards)
        self.failUnless(("Test Card 3", 2) in aCards)


if __name__ == "__main__":
    unittest.main()



# test_WriteArdbText.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2008 Neil Muller <drnlmuller+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test Writing a card set to an VEKN bbcode file"""

import time
from sutekh.tests.TestCore import SutekhTest
from sutekh.tests.core.test_PhysicalCardSet import make_set_1
from sutekh.core.CardSetHolder import CardSetWrapper
from sutekh.io.WriteVEKNForum import WriteVEKNForum
from sutekh.SutekhInfo import SutekhInfo
import unittest

VEKN_EXPECTED_1 = """[size=18][b]Deck Name : Test Set 1[/b][/size]
[b][u]Author :[/u][/b] A test author
[b][u]Description :[/u][/b]
A test comment

[size=18][u]Crypt [4 vampires] Capacity min: 3 max: 7 average: 5.00[/u][/size]
[table]
[tr][td]1x[/td][td][url=http://www.secretlibrary.info/?crypt=Siamese,+The]The Siamese[/url][/td][td][/td][td](7)[/td][td]:PRE: :SPI: :ani: :pro:[/td][td]   [/td][td]:ahri: Ahrimane[/td][td](group 2)[/td][/tr]
[tr][td]1x[/td][td][url=http://www.secretlibrary.info/?crypt=Alan+Sovereign+Adv]Alan Sovereign[/url][/td][td]Adv[/td][td](6)[/td][td]:AUS: :DOM: :for: :pre:[/td][td]   [/td][td]:vent: Ventrue[/td][td](group 3)[/td][/tr]
[tr][td]1x[/td][td][url=http://www.secretlibrary.info/?crypt=Abebe]Abebe[/url][/td][td][/td][td](4)[/td][td]:nec: :obf: :thn:[/td][td]   [/td][td]:same: Samedi[/td][td](group 4)[/td][/tr]
[tr][td]1x[/td][td][url=http://www.secretlibrary.info/?crypt=Inez+Nurse216+Villagrande]Inez "Nurse216" Villagrande[/url][/td][td][/td][td](3)[/td][td]:inn:[/td][td]   [/td][td]:inno: Innocent (Imbued)[/td][td](group 4)[/td][/tr]
[/table]
[size=18][u]Library [13 cards][/u][/size]
[b][u]Action [2][/u][/b]
 2x [url=http://www.secretlibrary.info/?lib=Abbot]Abbot[/url]

[b][u]Ally [1][/u][/b]
 1x [url=http://www.secretlibrary.info/?lib=Scapelli,+The+Family+Mechanic]Scapelli, The Family "Mechanic"[/url]

[b][u]Equipment [7][/u][/b]
 4x [url=http://www.secretlibrary.info/?lib=.44+Magnum].44 Magnum[/url]
 2x [url=http://www.secretlibrary.info/?lib=AK-47]AK-47[/url]
 1x [url=http://www.secretlibrary.info/?lib=Aaron's+Feeding+Razor]Aaron's Feeding Razor[/url]

[b][u]Master [3][/u][/b]
 2x [url=http://www.secretlibrary.info/?lib=Abombwe]Abombwe[/url]
 1x [url=http://www.secretlibrary.info/?lib=Path+of+Blood,+The]The Path of Blood[/url]


Recorded with : Sutekh %s [ DATE ]
""" % SutekhInfo.VERSION_STR


class VEKNForumWriterTests(SutekhTest):
    """class for the VEKN bbcode file writer tests"""

    def test_deck_writer(self):
        """Test VEKN bbcode file writing"""
        oPhysCardSet1 = make_set_1()

        sCurDate = time.strftime('[ %Y-%m-%d ]', time.localtime())

        # Check output
        oWriter = WriteVEKNForum()
        sData = self._round_trip_obj(oWriter, CardSetWrapper(oPhysCardSet1))
        sData = sData.replace(sCurDate, '[ DATE ]')

        self.assertEqual(sData, VEKN_EXPECTED_1)


if __name__ == "__main__":
    unittest.main()



# test_DataPack.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2011 Simon Cross <hodgestar+sutekh@gmail.com>
# GPL - see COPYING for details

"""Test the Data Pack utilities"""

import urllib
import unittest
from sutekh.tests.TestCore import SutekhTest
from sutekh.io.DataPack import find_data_pack

TEST_DATA = """
= User Documentation =

== A Heading ==

  * [GettingStarted Starting out with Sutekh]

== Data Packs ==

{{{
#!comment
A comment.
}}}

|| '''Description''' || '''Tag''' || '''File'''
|| Zip file of starter decks from Sabbat War to Heirs to the Blood || starters || [attachment:Starters_SW_to_HttB.zip:wiki:MiscWikiFiles Starters_SW_to_HttB.zip] ||
|| Zip file of userful rulings and rulebooks || rulebooks || [attachment:Rulebooks.zip:wiki:MiscWikiFiles Rulebooks.zip] ||
"""


class DataPackTest(SutekhTest):
    """Class for the data pack tests"""

    def test_find_data_pack(self):
        """Test finding data pack in sane documentation page"""
        sTempFileName = self._create_tmp_file()
        sTempUrl = 'file://%s' % urllib.pathname2url(sTempFileName)
        fFile = open(sTempFileName, 'w')
        fFile.write(TEST_DATA)
        fFile.close()

        self.assertEqual(find_data_pack('starters', sTempUrl),
                         "http://sourceforge.net/apps/trac/sutekh/"
                         "raw-attachment/wiki/MiscWikiFiles/"
                         "Starters_SW_to_HttB.zip")

        self.assertEqual(find_data_pack('rulebooks', sTempUrl),
                         "http://sourceforge.net/apps/trac/sutekh/"
                         "raw-attachment/wiki/MiscWikiFiles/"
                         "Rulebooks.zip")


if __name__ == "__main__":
    unittest.main()



# ELDBHTMLParser.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# ELDB HTML Deck Parser
# Copyright 2007 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

# pylint: disable-msg=C0301
# documentation, so line length ignored
"""Parser for ELDB HTML format.

   Example HTML:

   <TR><TD WIDTH=130>Deck Name:</TD><TD WIDTH=520>Osebo Preconstructed Starter Deck</TD></TR>
   <TR><TD WIDTH=130>Created By:</TD><TD WIDTH=520>L. Scott Johnson</TD></TR>
   <TR><TD WIDTH=130 VALIGN="top">Description:</TD><TD WIDTH=520>The Osebo Preconstructed Starter Deck from Legacies of Blood.</TD></TR>
   <TR><TD COLSPAN=2 WIDTH=650>&nbsp;</TD><TR>
   <TR><TD COLSPAN=2 WIDTH=650 BGCOLOR="#eeeeee">Crypt: (12 cards, Min: 12, Max: 36, Avg: 6.00)</TD></TR>
   <TR><TD COLSPAN=2 WIDTH=650>2&nbsp;&nbsp;<a href="http://www.white-wolf.com/vtes/index.php?line=Checklist_LegaciesOfBlood" class="textLink">Uzoma</a> ... </TD></TR>
   ...
   <TR><TD COLSPAN=2 WIDTH=650>2&nbsp;&nbsp;<a href="http://monger.vekn.org/showcard.html?ID=109" class="textLink">Blood Doll</a></TD></TR>
   """
# pylint: enable-msg=C0301

from sutekh.io.SutekhBaseHTMLParser import SutekhBaseHTMLParser, HolderState
import re


# State Classes
class Collecting(HolderState):
    """Default state - transitions to other states as needed"""

    def transition(self, sTag, dAttr):
        """Transition to CardItem of DeckInfoItem as needed."""
        if sTag == 'td' and dAttr.get('colspan') == '2':
            return CardItem(self._oHolder)
        elif sTag == 'td':
            return DeckInfoItem(self._oHolder)
        else:
            return self


class DeckInfoItem(HolderState):
    """States for the table rows describing the deck."""

    def transition(self, sTag, _dAttr):
        """Transition back to Collecting if needed"""
        if sTag == '/tr':
            aParts = self._sData.split(':', 1)

            if len(aParts) != 2:
                return Collecting(self._oHolder)

            sItem, sText = aParts

            if sItem == "Deck Name":
                self._oHolder.name = sText
            elif sItem == "Created By":
                self._oHolder.author = sText
            elif sItem == "Description":
                self._oHolder.comment = sText

            return Collecting(self._oHolder)
        else:
            return self


class CardItem(HolderState):
    """State for the table rows listing the cards in the deck."""
    _oCountRegex = re.compile(r'^[^0-9]*(?P<cnt>[0-9]+)[^0-9]*')

    def __init__(self, oHolder):
        super(CardItem, self).__init__(oHolder)
        self._iCnt = None

    def transition(self, sTag, _dAttr):
        """Extract card data and add it back to the CardSetHolder if possible,
           and transtion back to Collecting if needed."""
        if sTag == 'a':
            oMatch = self._oCountRegex.match(self._sData)
            if oMatch:
                self._iCnt = int(oMatch.group('cnt'))
            else:
                self._iCnt = 1
            self._sData = ""
            return self
        elif sTag == '/a':
            assert(self._iCnt is not None)
            sName = self._sData.strip()
            sName = sName.replace("`", "'")
            # No expansion info for these
            self._oHolder.add(self._iCnt, sName, None)
            self._iCnt = None
            self._sData = ""
            return Collecting(self._oHolder)
        elif sTag == '/tr':
            return Collecting(self._oHolder)
        else:
            return self


# Parser
class ELDBHTMLParser(SutekhBaseHTMLParser):
    """Actual Parser for the ELDB HTML files."""

    def __init__(self):
        """Create an ELDBHTMLParser.
           """
        # super __init__ will call rest, so need this
        self._oHolder = None  # Placeholder for later
        super(ELDBHTMLParser, self).__init__()
        # Don't need to set oState, since reset will do that

    def reset(self):
        """Reset the parser"""
        super(ELDBHTMLParser, self).reset()
        self._oState = Collecting(self._oHolder)

    def parse(self, fIn, oHolder):
        """Parse a file into the given holder"""
        self._oHolder = oHolder
        self.reset()
        for sLine in fIn:
            self.feed(sLine)



# __init__.py
# -*- coding: utf8 -*-
# vim:fileencoding=utf8 ai ts=4 sts=4 et sw=4
# Copyright 2007 Simon Cross <hodgestar@gmail.com>
# GPL - see COPYING for details

"""This is the sutekh test suite"""

from sutekh.core.SutekhObjects import TABLE_LIST, VersionTable
from sutekh.SutekhUtility import read_white_wolf_list, read_rulings, \
        refresh_tables
from sutekh.tests.TestData import TEST_CARD_LIST, TEST_RULINGS
from sutekh.tests.TestCore import SutekhTest, make_null_handler
from sutekh.io.WwFile import WwFile
from sqlobject import sqlhub, connectionForURI
import tempfile
import os


def _create_pkg_tmp_file(sData):
    """Create a temporary file for use in create_db"""
    (fTemp, sFilename) = tempfile.mkstemp()
    os.close(fTemp)

    if sData:
        fTmp = file(sFilename, "wb")
        fTmp.write(sData)
        fTmp.close()

    return sFilename


def create_db():
    """Create the database"""
    assert refresh_tables(TABLE_LIST, sqlhub.processConnection)

    sCardList = _create_pkg_tmp_file(TEST_CARD_LIST)
    sRulings = _create_pkg_tmp_file(TEST_RULINGS)

    oLogHandler = make_null_handler()
    read_white_wolf_list([WwFile(sCardList)], oLogHandler)
    read_rulings([WwFile(sRulings)], oLogHandler)

    os.remove(sRulings)
    os.remove(sCardList)


def setup_package():
    """Fill database before any of the tests are run"""
    # Get the database to use from the environment, defaulting to an
    # sqlite memory DB
    sDBUrl = os.getenv('SUTEKH_TEST_DB', "sqlite:///:memory:")
    oConn = connectionForURI(sDBUrl)
    sqlhub.processConnection = oConn
    SutekhTest.set_db_conn(oConn)

    create_db()


def teardown_package():
    """Cleanup the database after all the tests are done"""
    sSaveDB = os.getenv('SUTEKH_SAVE_DB', "No").lower()
    if sSaveDB != 'yes':
        for cCls in reversed(TABLE_LIST):
            cCls.dropTable(ifExists=True)
        VersionTable.dropTable(ifExists=True)
